/*-----------------------------------------------------------------------
 *      File:   CSSMAPI.H
 *
 * Copyright (c) 1995, 1996, 1997 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
/* 
 * WARNING: EXPORT RESTRICTED. 
 * This software listing contains cryptographic methods and technology. 
 * It is export restricted by the Office of Defense Trade Controls, United 
 * States Department of State and cannot be downloaded or otherwise 
 * exported or re-exported (i) into (or to a national or resident of) Cuba, 
 * Iraq, Libya, Yugoslavia, North Korea, Iran, Syria or any other country 
 * to which the US has embargoed goods; or (ii) to anyone on the US 
 * Treasury Department's list of Specially Designated Nationals or the US 
 * Commerce Department's Table of Denial Orders. By downloading or using 
 * this product, you are agreeing to the foregoing and you are representing 
 * and warranting that you are not located in, under the control of, or a 
 * national or resident of any such country or on any such list. 
 */ 

#ifndef _CSSMAPI_H
#define _CSSMAPI_H    

#include "cssmtype.h"

/* API Functions */
#ifdef __cplusplus
extern "C" {
#endif

/* CSSM functions */
CSSM_RETURN CSSMAPI CSSM_Init (uint32 CheckCompatibleVerMajor,
                               uint32 CheckCompatibleVerMinor,
                               const CSSM_API_MEMORY_FUNCS_PTR MemoryFuncs,
                               const void * Reserved);

CSSM_INFO_PTR CSSMAPI CSSM_GetInfo (void);

/* Context API */
CSSM_CC_HANDLE CSSMAPI CSSM_CSP_CreateKeyExchContext (CSSM_CSP_HANDLE CSPHandle,
                                                      uint32 AlgorithmID);

CSSM_CC_HANDLE CSSMAPI CSSM_CSP_CreateSignatureContext (
                                          CSSM_CSP_HANDLE CSPHandle,
                                          uint32 AlgorithmID,
                                          const CSSM_CRYPTO_DATA_PTR PassPhrase,
                                          const CSSM_KEY_PTR Key);

CSSM_CC_HANDLE CSSMAPI CSSM_CSP_CreateSymmetricContext (
                                            CSSM_CSP_HANDLE CSPHandle,
                                            uint32 AlgorithmID,
                                            uint32 Mode,
                                            const CSSM_KEY_PTR Key,
                                            const CSSM_DATA_PTR InitVector, 
                                            uint32 Padding,
                                            uint32 Rounds);

CSSM_CC_HANDLE CSSMAPI CSSM_CSP_CreateDigestContext (CSSM_CSP_HANDLE CSPHandle,
                                                     uint32 AlgorithmID);

CSSM_CC_HANDLE CSSMAPI CSSM_CSP_CreateMacContext (CSSM_CSP_HANDLE CSPHandle,
                                                  uint32 AlgorithmID,
                                                  const CSSM_KEY_PTR Key);

CSSM_CC_HANDLE CSSMAPI CSSM_CSP_CreateRandomGenContext (
                                            CSSM_CSP_HANDLE CSPHandle,
                                            uint32 AlgorithmID,
                                            const CSSM_CRYPTO_DATA_PTR Seed,
                                            uint32 Length);

CSSM_CC_HANDLE CSSMAPI CSSM_CSP_CreateUniqueIdContext (
                                            CSSM_CSP_HANDLE CSPHandle,
                                            uint32 AlgorithmID,
                                            const CSSM_CRYPTO_DATA_PTR Seed,
                                            uint32 Length);

CSSM_CC_HANDLE CSSMAPI CSSM_CSP_CreateAsymmetricContext (
                                          CSSM_CSP_HANDLE CSPHandle,
                                          uint32 AlgorithmID,
                                          const CSSM_CRYPTO_DATA_PTR PassPhrase,
                                          const CSSM_KEY_PTR Key,
                                          uint32 Padding,
                                          uint32 KeyMode);

CSSM_CC_HANDLE CSSMAPI CSSM_CSP_CreateKeyGenContext (
                                          CSSM_CSP_HANDLE CSPHandle,
                                          uint32 AlgorithmID,
                                          const CSSM_CRYPTO_DATA_PTR PassPhrase,
                                          uint32 KeySizeInBits,
                                          const CSSM_CRYPTO_DATA_PTR Seed,
                                          const CSSM_DATA_PTR Salt,
                                          const CSSM_DATA_PTR KeyLabel);

CSSM_CC_HANDLE CSSMAPI CSSM_CSP_CreateDeriveKeyContext (
                                        CSSM_CSP_HANDLE CSPHandle,
                                        uint32 AlgorithmID,
                                        CSSM_KEY_TYPE DeriveKeyType,
                                        uint32 DeriveKeyLength,
                                        uint32 IterationCount,
                                        const CSSM_CRYPTO_DATA_PTR Seed,
                                        const CSSM_CRYPTO_DATA_PTR PassPhrase);

CSSM_CC_HANDLE CSSMAPI CSSM_CSP_CreatePassThroughContext (
                                                  CSSM_CSP_HANDLE CSPHandle,
                                                  const CSSM_KEY_PTR Key,
                                                  const CSSM_DATA_PTR ParamBufs,
                                                  uint32 ParamBufCount);

CSSM_CONTEXT_PTR CSSMAPI CSSM_GetContext (CSSM_CC_HANDLE CCHandle);

CSSM_RETURN CSSMAPI CSSM_SetContext (CSSM_CC_HANDLE CCHandle,
                                     const CSSM_CONTEXT_PTR Context);

CSSM_RETURN CSSMAPI CSSM_DeleteContext (CSSM_CC_HANDLE CCHandle);

CSSM_CONTEXT_ATTRIBUTE_PTR CSSMAPI CSSM_GetContextAttribute (
                                            const CSSM_CONTEXT_PTR Context,
                                            uint32 AttributeType);

CSSM_RETURN CSSMAPI CSSM_FreeContext (CSSM_CONTEXT_PTR Context);
CSSM_RETURN CSSMAPI CSSM_UpdateContextAttributes (
                           CSSM_CC_HANDLE CCHandle,
                           uint32 NumberAttributes,
                           const CSSM_CONTEXT_ATTRIBUTE_PTR ContextAttributes);

CSSM_RETURN CSSMAPI CSSM_DeleteContextAttributes (
                           CSSM_CC_HANDLE CCHandle,
                           uint32 NumberAttributes,
                           const CSSM_CONTEXT_ATTRIBUTE_PTR ContextAttributes);


/* Cryptography API */
/* Cryptography Operations */

CSSM_RETURN CSSMAPI CSSM_QuerySize (CSSM_CC_HANDLE CCHandle,
                                    uint32 SizeOfInput,
                                    uint32 * ReqSizeOutBlock);

/*Returns a signature item */
CSSM_RETURN CSSMAPI CSSM_SignData (CSSM_CC_HANDLE CCHandle,
                                   const CSSM_DATA_PTR DataBufs,
                                   uint32 DataBufCount,
                                   CSSM_DATA_PTR Signature);

CSSM_RETURN CSSMAPI CSSM_SignDataInit (CSSM_CC_HANDLE CCHandle);

CSSM_RETURN CSSMAPI CSSM_SignDataUpdate (CSSM_CC_HANDLE CCHandle,
                                         const CSSM_DATA_PTR DataBufs,
                                         uint32 DataBufCount);

/* Returns a signature item */
CSSM_RETURN CSSMAPI CSSM_SignDataFinal (CSSM_CC_HANDLE CCHandle,
                                        CSSM_DATA_PTR Signature);

CSSM_BOOL CSSMAPI CSSM_VerifyData (CSSM_CC_HANDLE CCHandle,
                                   const CSSM_DATA_PTR DataBufs,
                                   uint32 DataBufCount,
                                   const CSSM_DATA_PTR Signature);

CSSM_RETURN CSSMAPI CSSM_VerifyDataInit (CSSM_CC_HANDLE CCHandle,
                                         const CSSM_DATA_PTR Signature);

CSSM_RETURN CSSMAPI CSSM_VerifyDataUpdate (CSSM_CC_HANDLE CCHandle,
                                           const CSSM_DATA_PTR DataBufs,
                                           uint32 DataBufCount);

CSSM_BOOL CSSMAPI CSSM_VerifyDataFinal (CSSM_CC_HANDLE CCHandle);

/* Returns a digest item */
CSSM_RETURN CSSMAPI CSSM_DigestData (CSSM_CC_HANDLE CCHandle,
                                     const CSSM_DATA_PTR DataBufs,
                                     uint32 DataBufCount,
                                     CSSM_DATA_PTR Digest);

CSSM_RETURN CSSMAPI CSSM_DigestDataInit (CSSM_CC_HANDLE CCHandle);

CSSM_RETURN CSSMAPI CSSM_DigestDataUpdate (CSSM_CC_HANDLE CCHandle,
                                           const CSSM_DATA_PTR DataBufs,
                                           uint32 DataBufCount);

CSSM_CC_HANDLE CSSMAPI CSSM_DigestDataClone (CSSM_CC_HANDLE CCHandle);

/* Returns a digest item */
CSSM_RETURN CSSMAPI CSSM_DigestDataFinal (CSSM_CC_HANDLE CCHandle,
                                          CSSM_DATA_PTR Digest);

/* Returns a mac data */
CSSM_RETURN CSSMAPI CSSM_GenerateMac (CSSM_CC_HANDLE CCHandle,
                                      const CSSM_DATA_PTR DataBufs,
                                      uint32 DataBufCount,
                                      CSSM_DATA_PTR Mac);

CSSM_RETURN CSSMAPI CSSM_GenerateMacInit (CSSM_CC_HANDLE CCHandle);

CSSM_RETURN CSSMAPI CSSM_GenerateMacUpdate (CSSM_CC_HANDLE CCHandle,
                                            const CSSM_DATA_PTR DataBufs,
                                            uint32 DataBufCount);

/* Returns a mac data */
CSSM_RETURN CSSMAPI CSSM_GenerateMacFinal (CSSM_CC_HANDLE CCHandle,
                                           CSSM_DATA_PTR Mac);

/* Returns the encrypted data */
CSSM_RETURN CSSMAPI CSSM_EncryptData (CSSM_CC_HANDLE CCHandle,
                                      const CSSM_DATA_PTR ClearBufs,
                                      uint32 ClearBufCount,
                                      CSSM_DATA_PTR CipherBufs,
                                      uint32 CipherBufCount,
                                      uint32 *bytesEncrypted,
                                      CSSM_DATA_PTR RemData);

CSSM_RETURN CSSMAPI CSSM_EncryptDataInit (CSSM_CC_HANDLE CCHandle);

CSSM_RETURN CSSMAPI CSSM_EncryptDataUpdate (CSSM_CC_HANDLE CCHandle,
                                            const CSSM_DATA_PTR ClearBufs,
                                            uint32 ClearBufCount,
                                            CSSM_DATA_PTR CipherBufs,
                                            uint32 CipherBufCount,
                                            uint32 *bytesEncrypted);

/* Returns the encrypted data */
CSSM_RETURN CSSMAPI CSSM_EncryptDataFinal (CSSM_CC_HANDLE CCHandle,
                                           CSSM_DATA_PTR RemData);

/* Returns the decrypted data */
CSSM_RETURN CSSMAPI CSSM_DecryptData (const CSSM_CC_HANDLE CCHandle,
                                      const CSSM_DATA_PTR CipherBufs,
                                      uint32 CipherBufCount,
                                      CSSM_DATA_PTR ClearBufs,
                                      uint32 ClearBufCount,
                                      uint32 *bytesDecrypted,
                                      CSSM_DATA_PTR RemData);

CSSM_RETURN CSSMAPI CSSM_DecryptDataInit (CSSM_CC_HANDLE CCHandle);

CSSM_RETURN CSSMAPI CSSM_DecryptDataUpdate (CSSM_CC_HANDLE CCHandle,
                                            const CSSM_DATA_PTR CipherBufs,
                                            uint32 CipherBufCount,
                                            CSSM_DATA_PTR ClearBufs,
                                            uint32 ClearBufCount,
                                            uint32 *bytesDecrypted);
/* Returns the decrypted data */
CSSM_RETURN CSSMAPI CSSM_DecryptDataFinal (CSSM_CC_HANDLE CCHandle,
                                           CSSM_DATA_PTR RemData);

/* Returns the symmetric key */
CSSM_RETURN CSSMAPI CSSM_GenerateKey (CSSM_CC_HANDLE CCHandle,
                                      CSSM_BOOL StoreKey,
                                      CSSM_KEY_PTR Key);

/* Returns the asymmetric key pair */
CSSM_RETURN CSSMAPI CSSM_GenerateKeyPair (CSSM_CC_HANDLE CCHandle,
                                          CSSM_BOOL StorePublicKey,
                                          CSSM_KEY_PTR PublicKey,
                                          CSSM_KEY_PTR PrivateKey);

/* Returns the random data */
CSSM_RETURN CSSMAPI CSSM_GenerateRandom (CSSM_CC_HANDLE CCHandle,
                                         CSSM_DATA_PTR RandomNumber);

/* Returns the unique id */
CSSM_RETURN CSSMAPI CSSM_GenerateUniqueId (CSSM_CC_HANDLE CCHandle,
                                           CSSM_DATA_PTR UniqueID);

CSSM_RETURN CSSMAPI CSSM_KeyExchGenParam (CSSM_CC_HANDLE CCHandle,
                                          uint32 ParamBits,
                                          CSSM_DATA_PTR Param);

CSSM_RETURN CSSMAPI CSSM_KeyExchPhase1 (CSSM_CC_HANDLE CCHandle, 
                                        const CSSM_DATA_PTR Param,
                                        CSSM_DATA_PTR Param1);

/* Returns the key */
CSSM_RETURN CSSMAPI CSSM_KeyExchPhase2 (CSSM_CC_HANDLE CCHandle, 
                                        const CSSM_DATA_PTR Param1,
                                        CSSM_KEY_PTR ExchangedKey);


CSSM_RETURN CSSMAPI CSSM_WrapKey (CSSM_CC_HANDLE CCHandle,
                                  const CSSM_CRYPTO_DATA_PTR PassPhrase, 
                                  CSSM_KEY_PTR Key,
                                  CSSM_WRAP_KEY_PTR WrappedKey);

CSSM_RETURN CSSMAPI CSSM_UnwrapKey (CSSM_CC_HANDLE CCHandle,
                                    const CSSM_CRYPTO_DATA_PTR NewPassPhrase,
                                    const CSSM_WRAP_KEY_PTR WrappedKey,
                                    CSSM_BOOL StoreKey,
                                    CSSM_KEY_PTR UnwrappedKey);

CSSM_RETURN CSSMAPI CSSM_DeriveKey (CSSM_CC_HANDLE CCHandle,
                                    const CSSM_KEY_PTR BaseKey,
                                    CSSM_DATA_PTR Param,
                                    CSSM_BOOL StoreKey,
                                    CSSM_KEY_PTR DerivedKey);

CSSM_RETURN CSSMAPI CSSM_CSP_PassThrough (CSSM_CC_HANDLE CCHandle, 
                                          uint32 PassThroughId,
                                          const CSSM_DATA_PTR InData,
                                          CSSM_DATA_PTR OutData);

/* CSP Loging Functions */
CSSM_RETURN CSSMAPI CSSM_CSP_Login (CSSM_CSP_HANDLE CSPHandle,
                                    const CSSM_CRYPTO_DATA_PTR Password,
                                    const CSSM_DATA_PTR pReserved);

CSSM_RETURN CSSMAPI CSSM_CSP_Logout (CSSM_CSP_HANDLE CSPHandle);

CSSM_RETURN CSSMAPI CSSM_CSP_ChangeLoginPassword (
                                    CSSM_CSP_HANDLE CSPHandle,
                                    const CSSM_CRYPTO_DATA_PTR OldPassword,
                                    const CSSM_CRYPTO_DATA_PTR NewPassword);

/* CSP Registry Functions */
CSSM_RETURN CSSMAPI CSSM_CSP_Install (const char *CSPName, 
                                      const char *CSPFileName, 
                                      const char *CSPPathName,
                                      const CSSM_GUID_PTR GUID,
                                      const CSSM_CSPINFO_PTR CSPInfo,
                                      const void *Reserved1,
                                      const CSSM_DATA_PTR Reserved2);

CSSM_RETURN CSSMAPI CSSM_CSP_Uninstall (const CSSM_GUID_PTR GUID);

CSSM_LIST_PTR CSSMAPI CSSM_CSP_ListModules (void);

CSSM_CSP_HANDLE CSSMAPI CSSM_CSP_Attach (
                                    const CSSM_GUID_PTR GUID,
                                    uint32 CheckCompatibleVerMajor,
                                    uint32 CheckCompatibleVerMinor,
                                    const CSSM_API_MEMORY_FUNCS_PTR MemoryFuncs,
                                    uint32 SlotID,
                                    uint32 SessionFlags,
                                    uint32 Application,
                                    const CSSM_NOTIFY_CALLBACK Notification,
                                    const void * Reserved);

CSSM_RETURN CSSMAPI CSSM_CSP_Detach (CSSM_CSP_HANDLE CSPHandle);

CSSM_CSPINFO_PTR CSSMAPI CSSM_CSP_GetInfo (const CSSM_GUID_PTR GUID,
                                           CSSM_BOOL CompleteCapabilitiesOnly,
                                           uint32 *NumberOfInfos);

CSSM_RETURN CSSMAPI CSSM_CSP_FreeInfo (CSSM_CSPINFO_PTR CSPInfos,
                                       uint32 NumberOfInfos);

/* Trust Policy API */
/* Trust Policy Operations */
CSSM_BOOL CSSMAPI CSSM_TP_CertVerify (CSSM_TP_HANDLE TPHandle, 
                                      CSSM_CL_HANDLE CLHandle, 
                                      CSSM_DL_HANDLE DLHandle, 
                                      CSSM_DB_HANDLE DBHandle, 
                                      CSSM_CC_HANDLE CCHandle, 
                                      const CSSM_DATA_PTR SubjectCert, 
                                      const CSSM_DATA_PTR SignerCert, 
                                      const CSSM_FIELD_PTR VerifyScope,
                                      uint32 ScopeSize);

/* Returns the signed certificate */
CSSM_DATA_PTR CSSMAPI CSSM_TP_CertSign (CSSM_TP_HANDLE TPHandle, 
                                        CSSM_CL_HANDLE CLHandle, 
                                        CSSM_DL_HANDLE DLHandle, 
                                        CSSM_DB_HANDLE DBHandle, 
                                        CSSM_CC_HANDLE CCHandle, 
                                        const CSSM_DATA_PTR SubjectCert, 
                                        const CSSM_DATA_PTR SignerCert, 
                                        CSSM_FIELD_PTR SignScope, 
                                        uint32 ScopeSize);

/* Returns the new CRL */
CSSM_DATA_PTR CSSMAPI CSSM_TP_CertRevoke (CSSM_TP_HANDLE TPHandle, 
                                          CSSM_CL_HANDLE CLHandle, 
                                          CSSM_DL_HANDLE DLHandle, 
                                          CSSM_DB_HANDLE DBHandle, 
                                          CSSM_CC_HANDLE CCHandle,  
                                          const CSSM_DATA_PTR OldCrl, 
                                          const CSSM_DATA_PTR SubjectCert, 
                                          const CSSM_DATA_PTR RevokerCert, 
                                          CSSM_REVOKE_REASON Reason);

CSSM_BOOL CSSMAPI CSSM_TP_CrlVerify (CSSM_TP_HANDLE TPHandle, 
                                     CSSM_CL_HANDLE CLHandle, 
                                     CSSM_DL_HANDLE DLHandle, 
                                     CSSM_DB_HANDLE DBHandle, 
                                     CSSM_CC_HANDLE CCHandle,  
                                     const CSSM_DATA_PTR SubjectCrl,
                                     const CSSM_DATA_PTR SignerCert, 
                                     const CSSM_FIELD_PTR VerifyScope,
                                     uint32 ScopeSize);

/* Returns the signed CRL */
CSSM_DATA_PTR CSSMAPI CSSM_TP_CrlSign (CSSM_TP_HANDLE TPHandle, 
                                       CSSM_CL_HANDLE CLHandle, 
                                       CSSM_DL_HANDLE DLHandle, 
                                       CSSM_DB_HANDLE DBHandle, 
                                       CSSM_CC_HANDLE CCHandle,  
                                       const CSSM_DATA_PTR SubjectCrl, 
                                       const CSSM_DATA_PTR SignerCert, 
                                       const CSSM_FIELD_PTR SignScope,
                                       uint32 ScopeSize);

CSSM_RETURN CSSMAPI CSSM_TP_ApplyCrlToDb (CSSM_TP_HANDLE TPHandle, 
                                          CSSM_CL_HANDLE CLHandle, 
                                          CSSM_DL_HANDLE DLHandle, 
                                          CSSM_DB_HANDLE DBHandle, 
                                          const CSSM_DATA_PTR Crl);

CSSM_RETURN CSSMAPI CSSM_TP_VerifyAction (CSSM_TP_HANDLE TPHandle, 
                                          CSSM_CL_HANDLE CLHandle, 
                                          CSSM_DL_HANDLE DLHandle, 
                                          CSSM_DB_HANDLE DBHandle, 
                                          CSSM_CC_HANDLE CCHandle,  
                                          CSSM_TP_ACTION Action, 
                                          const CSSM_DATA_PTR Data, 
                                          const CSSM_DATA_PTR Cert);

CSSM_DATA_PTR CSSMAPI CSSM_TP_PassThrough (CSSM_TP_HANDLE TPHandle, 
                                           CSSM_CL_HANDLE CLHandle, 
                                           CSSM_DL_HANDLE DLHandle, 
                                           CSSM_DB_HANDLE DBHandle, 
                                           CSSM_CC_HANDLE CCHandle, 
                                           uint32 PassThroughId,
                                           const CSSM_DATA_PTR InputParams);

/* Trust Policy Registry Functions */
CSSM_RETURN CSSMAPI CSSM_TP_Install (const char *TPName, 
                                     const char *TPFileName, 
                                     const char *TPPathName,
                                     const CSSM_GUID_PTR GUID,
                                     const CSSM_TPINFO_PTR TPInfo,
                                     const void *Reserved1,
                                     const CSSM_DATA_PTR Reserved2);

CSSM_RETURN CSSMAPI CSSM_TP_Uninstall (const CSSM_GUID_PTR GUID);

CSSM_LIST_PTR CSSMAPI CSSM_TP_ListModules(void);

CSSM_TP_HANDLE CSSMAPI CSSM_TP_Attach (
                                 const CSSM_GUID_PTR GUID,
                                 uint32 CheckCompatibleVerMajor,
                                 uint32 CheckCompatibleVerMinor,
                                 const CSSM_API_MEMORY_FUNCS_PTR MemoryFuncs,
                                 uint32 Application,
                                 const CSSM_NOTIFY_CALLBACK Notification,
                                 const void *Reserved);

CSSM_RETURN CSSMAPI CSSM_TP_Detach (CSSM_TP_HANDLE TPHandle);

CSSM_TPINFO_PTR CSSMAPI CSSM_TP_GetInfo (const CSSM_GUID_PTR GUID);

CSSM_RETURN CSSMAPI CSSM_TP_FreeInfo (CSSM_TPINFO_PTR TPInfo);


/* Certificate Library Operations */

/* Returns the signed certificate */
CSSM_DATA_PTR CSSMAPI CSSM_CL_CertSign (CSSM_CL_HANDLE CLHandle, 
                                        CSSM_CC_HANDLE CCHandle, 
                                        const CSSM_DATA_PTR SubjectCert, 
                                        const CSSM_DATA_PTR SignerCert, 
                                        const CSSM_FIELD_PTR SignScope, 
                                        uint32 ScopeSize);

/* Returns the unsigned certificate */
CSSM_DATA_PTR CSSMAPI CSSM_CL_CertUnsign (CSSM_CL_HANDLE CLHandle, 
                                          CSSM_CC_HANDLE CCHandle, 
                                          const CSSM_DATA_PTR SubjectCert, 
                                          const CSSM_DATA_PTR SignerCert,
                                          const CSSM_FIELD_PTR SignScope,
                                          uint32 ScopeSize);

CSSM_BOOL CSSMAPI CSSM_CL_CertVerify (CSSM_CL_HANDLE CLHandle, 
                                      CSSM_CC_HANDLE CCHandle, 
                                      const CSSM_DATA_PTR SubjectCert, 
                                      const CSSM_DATA_PTR SignerCert, 
                                      const CSSM_FIELD_PTR VerifyScope,
                                      uint32 ScopeSize);

/* Returns the new certificate */
CSSM_DATA_PTR CSSMAPI CSSM_CL_CertCreate (CSSM_CL_HANDLE CLHandle, 
                                          const CSSM_FIELD_PTR CertTemplate, 
                                          uint32 NumberOfFields);

CSSM_FIELD_PTR CSSMAPI CSSM_CL_CertView (CSSM_CL_HANDLE CLHandle, 
                                         const CSSM_DATA_PTR Cert, 
                                         uint32 *NumberOfFields);

/* Returns the field value */
CSSM_DATA_PTR CSSMAPI CSSM_CL_CertGetFirstFieldValue (
                                                 CSSM_CL_HANDLE CLHandle, 
                                                 const CSSM_DATA_PTR Cert, 
                                                 const CSSM_OID_PTR CertField,
                                                 CSSM_HANDLE_PTR ResultsHandle,
                                                 uint32 *NumberOfMatchedFields);

/* Returns the field value */
CSSM_DATA_PTR CSSMAPI CSSM_CL_CertGetNextFieldValue (CSSM_CL_HANDLE CLHandle, 
                                                     CSSM_HANDLE ResultsHandle);

CSSM_RETURN CSSMAPI CSSM_CL_CertAbortQuery (CSSM_CL_HANDLE CLHandle, 
                                            CSSM_HANDLE ResultsHandle);

CSSM_KEY_PTR CSSMAPI CSSM_CL_CertGetKeyInfo (CSSM_CL_HANDLE CLHandle, 
                                             const CSSM_DATA_PTR Cert);

CSSM_FIELD_PTR CSSMAPI CSSM_CL_CertGetAllFields (CSSM_CL_HANDLE CLHandle,
                                                 const CSSM_DATA_PTR Cert,
                                                 uint32 *NumberOfFields);

/* Cert Group Functions */
/* Returns the constructed cert group */
CSSM_CERTGROUP_PTR CSSMAPI CSSM_CL_CertGroupConstruct (
                                               CSSM_CL_HANDLE CLHandle,
                                               CSSM_CERTGROUP_PTR CertGroupFrag,
                                               CSSM_DB_LIST_PTR DBList);

/* Returns the pruned cert group */
CSSM_CERTGROUP_PTR CSSMAPI CSSM_CL_CertGroupPrune (
                                           CSSM_CL_HANDLE CLHandle,
                                           CSSM_CERTGROUP_PTR OrderedCertGroup,
                                           CSSM_DB_LIST_PTR DBList);

CSSM_BOOL CSSMAPI CSSM_CL_CertGroupVerify (CSSM_CL_HANDLE CLHandle,
                                           CSSM_CSP_HANDLE CSPHandle,
                                           CSSM_CERTGROUP_PTR OrderedCertGroup,
                                           const CSSM_FIELD_PTR VerifyScope,
                                           uint32 ScopeSize,
                                           CSSM_DB_LIST_PTR DBList,
                                           CSSM_CERTGROUP_PTR RootCerts);

/* Returns the certificate in the native cert format */
CSSM_DATA_PTR CSSMAPI CSSM_CL_CertImport (CSSM_CL_HANDLE CLHandle, 
                                          CSSM_CERT_TYPE ForeignCertType, 
                                          const CSSM_DATA_PTR ForeignCert); 

/* Returns the certificate in the foreign format */
CSSM_DATA_PTR CSSMAPI CSSM_CL_CertExport (CSSM_CL_HANDLE CLHandle, 
                                          CSSM_CERT_TYPE TargetCertType, 
                                          const CSSM_DATA_PTR NativeCert);

CSSM_OID_PTR CSSMAPI CSSM_CL_CertDescribeFormat (CSSM_CL_HANDLE CLHandle,
                                                 uint32 *NumberOfFields);

/* Returns the new CRL */
CSSM_DATA_PTR CSSMAPI CSSM_CL_CrlCreate (CSSM_CL_HANDLE CLHandle); 

/* Returns the updated CRL */
CSSM_DATA_PTR CSSMAPI CSSM_CL_CrlAddCert (CSSM_CL_HANDLE CLHandle, 
                                          CSSM_CC_HANDLE CCHandle, 
                                          const CSSM_DATA_PTR Cert, 
                                          const CSSM_DATA_PTR RevokerCert, 
                                          CSSM_REVOKE_REASON RevokeReason, 
                                          const CSSM_DATA_PTR OldCrl);

/* Returns the updated CRL */
CSSM_DATA_PTR CSSMAPI CSSM_CL_CrlRemoveCert (CSSM_CL_HANDLE CLHandle, 
                                             const CSSM_DATA_PTR Cert, 
                                             const CSSM_DATA_PTR OldCrl);

/* Returns the signed CRL */
CSSM_DATA_PTR CSSMAPI CSSM_CL_CrlSign (CSSM_CL_HANDLE CLHandle, 
                                       CSSM_CC_HANDLE CCHandle, 
                                       const CSSM_DATA_PTR UnsignedCrl, 
                                       const CSSM_DATA_PTR SignerCert, 
                                       const CSSM_FIELD_PTR SignScope,
                                       uint32 ScopeSize);

CSSM_BOOL CSSMAPI CSSM_CL_CrlVerify (CSSM_CL_HANDLE CLHandle, 
                                     CSSM_CC_HANDLE CCHandle, 
                                     const CSSM_DATA_PTR SubjectCrl, 
                                     const CSSM_DATA_PTR SignerCert,
                                     const CSSM_FIELD_PTR VerifyScope,
                                     uint32 ScopeSize);

CSSM_BOOL CSSMAPI CSSM_CL_IsCertInCrl (CSSM_CL_HANDLE CLHandle, 
                                       const CSSM_DATA_PTR Cert, 
                                       const CSSM_DATA_PTR Crl);

/* Returns the first item in the CRL */
CSSM_DATA_PTR CSSMAPI CSSM_CL_CrlGetFirstFieldValue (
                                               CSSM_CL_HANDLE CLHandle, 
                                               const CSSM_DATA_PTR Crl,
                                               const CSSM_OID_PTR CrlField,
                                               CSSM_HANDLE_PTR ResultsHandle,
                                               uint32 *NumberOfMatchedFields);

/* Returns the next item in the CRL */
CSSM_DATA_PTR CSSMAPI CSSM_CL_CrlGetNextFieldValue (CSSM_CL_HANDLE CLHandle, 
                                                    CSSM_HANDLE ResultsHandle);

CSSM_RETURN CSSMAPI CSSM_CL_CrlAbortQuery (CSSM_CL_HANDLE CLHandle, 
                                           CSSM_HANDLE ResultsHandle);

CSSM_OID_PTR CSSMAPI CSSM_CL_CrlDescribeFormat (CSSM_CL_HANDLE CLHandle,
                                                uint32 *NumberOfFields);

CSSM_DATA_PTR CSSMAPI CSSM_CL_PassThrough (CSSM_CL_HANDLE CLHandle, 
                                           CSSM_CC_HANDLE CCHandle, 
                                           uint32 PassThroughId, 
                                           const CSSM_DATA_PTR InputParams);

/* Certificate Library Registry Functions */
CSSM_RETURN CSSMAPI CSSM_CL_Install (const char *CLName, 
                                     const char *CLFileName, 
                                     const char *CLPathName,
                                     const CSSM_GUID_PTR GUID,
                                     const CSSM_CLINFO_PTR CLInfo,
                                     const void *Reserved1,
                                     const CSSM_DATA_PTR Reserved2);

CSSM_RETURN CSSMAPI CSSM_CL_Uninstall (const CSSM_GUID_PTR GUID);

CSSM_LIST_PTR CSSMAPI CSSM_CL_ListModules (void);

CSSM_LIST_PTR CSSMAPI CSSM_CL_ListModulesForCertType (CSSM_CERT_TYPE CertType);

CSSM_CL_HANDLE CSSMAPI CSSM_CL_Attach (
                                 const CSSM_GUID_PTR GUID,
                                 uint32 CheckCompatibleVerMajor,
                                 uint32 CheckCompatibleVerMinor,
                                 const CSSM_API_MEMORY_FUNCS_PTR MemoryFuncs,
                                 uint32 Application,
                                 const CSSM_NOTIFY_CALLBACK Notification,
                                 const void *Reserved);

CSSM_RETURN CSSMAPI CSSM_CL_Detach (CSSM_CL_HANDLE CLHandle);

CSSM_CLINFO_PTR CSSMAPI CSSM_CL_GetInfo (const CSSM_GUID_PTR GUID);

CSSM_RETURN CSSMAPI CSSM_CL_FreeInfo (CSSM_CLINFO_PTR CLInfo);


/* Database Library Operations */

CSSM_DB_HANDLE CSSMAPI CSSM_DL_DbOpen (CSSM_DL_HANDLE DLHandle,
                                       const char *DbName);

CSSM_RETURN CSSMAPI CSSM_DL_DbClose (CSSM_DL_HANDLE DLHandle,
                                     CSSM_DB_HANDLE DBHandle);


CSSM_DB_HANDLE CSSMAPI CSSM_DL_DbCreate (CSSM_DL_HANDLE DLHandle,
                                         CSSM_TP_HANDLE TPHandle, 
                                         const char *DbName, 
                                         CSSM_MODULE_HANDLE_PTR ModuleHandles,
                                         CSSM_DBINFO_PTR DataStoreSchema);

CSSM_RETURN CSSMAPI CSSM_DL_DbDelete (CSSM_DL_HANDLE DLHandle,
                                      const char *DbName);

CSSM_RETURN CSSMAPI CSSM_DL_DataInsert (CSSM_DL_HANDLE DLHandle,
                                        CSSM_DB_HANDLE DBHandle,
                                        CSSM_MODULE_HANDLE_PTR  ModuleHandles,
                                        CSSM_DATA_RECORD_TYPE DataRecordType,
                                        CSSM_INDEX_RECORD_PTR IndexRecord,
                                        const CSSM_DATA_PTR DataRecord);

CSSM_RETURN CSSMAPI CSSM_DL_DataDelete (CSSM_DL_HANDLE DLHandle,
                                        CSSM_DB_HANDLE DBHandle,
                                        CSSM_MODULE_HANDLE_PTR  ModuleHandles,
                                        CSSM_DATA_RECORD_TYPE DataRecordType,
                                        CSSM_INDEX_RECORD_PTR IndexRecord,
                                        const CSSM_DATA_PTR DataRecord);

CSSM_DATA_PTR CSSMAPI CSSM_DL_DataGetFirst (CSSM_DL_HANDLE DLHandle,
                                            CSSM_DB_HANDLE DBHandle, 
                                            CSSM_DATA_RECORD_TYPE DataRecordType,
                                            CSSM_QUERY_TAG QueryTag,
                                            CSSM_QUERY_PTR Query,
                                            CSSM_HANDLE_PTR ResultsHandle,
                                            uint32 *NumberOfMatchedDataRecords);

CSSM_DATA_PTR CSSMAPI CSSM_DL_DataGetNext (
                                      CSSM_DL_HANDLE DLHandle,
                                      CSSM_DB_HANDLE DBHandle,
                                      CSSM_HANDLE ResultsHandle, 
                                      CSSM_DATA_RECORD_TYPE_PTR DataRecordType);

CSSM_RETURN CSSMAPI CSSM_DL_DataAbortQuery (CSSM_DL_HANDLE DLHandle,
                                            CSSM_DB_HANDLE DBHandle,
                                            CSSM_HANDLE ResultsHandle);

CSSM_RETURN CSSMAPI CSSM_DL_CertInsert (CSSM_DL_HANDLE DLHandle,
                                        CSSM_DB_HANDLE DBHandle,
                                        CSSM_CL_HANDLE CLHandle,
                                        const CSSM_DATA_PTR Cert);

CSSM_RETURN CSSMAPI CSSM_DL_CertDelete (CSSM_DL_HANDLE DLHandle,
                                        CSSM_DB_HANDLE DBHandle,
                                        CSSM_CL_HANDLE CLHandle,
                                        const CSSM_DATA_PTR Cert);

CSSM_RETURN CSSMAPI CSSM_DL_CertRevoke (CSSM_DL_HANDLE DLHandle,
                                        CSSM_DB_HANDLE DBHandle,
                                        const CSSM_DATA_PTR CertToBeRevoked);

/* Returns the first certificate in the database */
/* found using the selection predicate */
CSSM_DATA_PTR CSSMAPI CSSM_DL_CertGetFirst (
                               CSSM_DL_HANDLE DLHandle, 
                               CSSM_DB_HANDLE DBHandle, 
                               CSSM_SELECTION_PREDICATE_PTR SelectionPredicate, 
                               uint32 SizeSelectionPredicate,
                               CSSM_DB_CONJUNCTIVE Conjunctive,
                               CSSM_HANDLE_PTR ResultsHandle,
                               uint32 *NumberOfMatchedCerts);

/* Returns the next certificate in the database */
/* found using the previously given selection predicate */
CSSM_DATA_PTR CSSMAPI CSSM_DL_CertGetNext (CSSM_DL_HANDLE DLHandle, 
                                           CSSM_DB_HANDLE DBHandle, 
                                           CSSM_HANDLE ResultsHandle);

CSSM_RETURN CSSMAPI CSSM_DL_CertAbortQuery (CSSM_DL_HANDLE DLHandle, 
                                            CSSM_DB_HANDLE DBHandle,
                                            CSSM_HANDLE ResultsHandle);

CSSM_RETURN CSSMAPI CSSM_DL_CrlInsert (CSSM_DL_HANDLE DLHandle, 
                                       CSSM_DB_HANDLE DBHandle, 
                                       CSSM_CL_HANDLE CLHandle,
                                       const CSSM_DATA_PTR Crl); 

CSSM_RETURN CSSMAPI CSSM_DL_CrlDelete (CSSM_DL_HANDLE DLHandle, 
                                       CSSM_DB_HANDLE DBHandle,
                                       CSSM_CL_HANDLE CLHandle,
                                       const CSSM_DATA_PTR Crl);

/* Returns the first CRL in the database */
/*  found using the selection predicate */
CSSM_DATA_PTR CSSMAPI CSSM_DL_CrlGetFirst (
                               CSSM_DL_HANDLE DLHandle, 
                               CSSM_DB_HANDLE DBHandle, 
                               CSSM_SELECTION_PREDICATE_PTR SelectionPredicate, 
                               uint32 SizeSelectionPredicate,
                               CSSM_DB_CONJUNCTIVE Conjunctive,
                               CSSM_HANDLE_PTR ResultsHandle,
                               uint32 *NumberOfMatchedCrls);

/* Returns the next CRL in the database */
/* found using the previously given selection predicate */
CSSM_DATA_PTR CSSMAPI CSSM_DL_CrlGetNext (CSSM_DL_HANDLE DLHandle, 
                                          CSSM_DB_HANDLE DBHandle, 
                                          CSSM_HANDLE ResultsHandle);
CSSM_RETURN CSSMAPI CSSM_DL_CrlAbortQuery (CSSM_DL_HANDLE DLHandle, 
                                           CSSM_DB_HANDLE DBHandle,
                                           CSSM_HANDLE ResultsHandle);

CSSM_RETURN CSSMAPI CSSM_DL_DbImport (CSSM_DL_HANDLE DLHandle,
                                      const char *DbDestLogicalName,
                                      const char *DbSrcFileName);

CSSM_RETURN CSSMAPI CSSM_DL_DbExport (CSSM_DL_HANDLE DLHandle,
                                      const char *DbSrcLogicalName,
                                      const char *DbDestFileName);
CSSM_RETURN CSSMAPI  CSSM_DL_DbSetInfo (CSSM_DL_HANDLE DLHandle, 
                                        const char* DbName
                                        CSSM_DBINFO_PTR DbInfo);

CSSM_DBINFO_PTR CSSMAPI CSSM_DL_DbGetInfo (CSSM_DL_HANDLE DLHandle, 
                                           const char* DbName);

CSSM_RETURN CSSMAPI CSSM_DL_FreeDbInfo (CSSM_DBNFO_PTR DbInfo,
                                        uint32 NumberOfDbInfos);

CSSM_NAME_LIST_PTR CSSMAPI DL_GetDbHandleToName(CSSM_DL_HANDLE DLHandle, 
                                                CSSM_DB_HANDLE DbHandle); 

CSSM_DATA_PTR CSSMAPI CSSM_DL_PassThrough (CSSM_DL_HANDLE DLHandle, 
                                           CSSM_DB_HANDLE DBHandle, 
                                           uint32 PassThroughId,
                                           const CSSM_DATA_PTR InputParams); 

/* Database Library Registry Functions */

CSSM_RETURN CSSMAPI CSSM_DL_Install (const char *DLName, 
                                     const char *DLFileName, 
                                     const char *DLPathName,
                                     const CSSM_GUID_PTR GUID,
                                     const CSSM_DLINFO_PTR DLInfo,
                                     const void *Reserved1,
                                     const CSSM_DATA_PTR Reserved2);

CSSM_RETURN CSSMAPI CSSM_DL_Uninstall (const CSSM_GUID_PTR GUID);

CSSM_LIST_PTR CSSMAPI CSSM_DL_ListModules(void);

CSSM_DL_HANDLE CSSMAPI CSSM_DL_Attach (
                                 const CSSM_GUID_PTR GUID,
                                 uint32 CheckCompatibleVerMajor,
                                 uint32 CheckCompatibleVerMinor,
                                 const CSSM_API_MEMORY_FUNCS_PTR MemoryFuncs,
                                 const uint32 DeviceID,
                                 const uint32 DeviceAccessFlags,
                                 uint32 Application,
                                 const CSSM_NOTIFY_CALLBACK Notification,
                                 const void *Reserved);

CSSM_RETURN CSSMAPI CSSM_DL_Detach (CSSM_DL_HANDLE DLHandle);

CSSM_DLINFO_PTR CSSMAPI CSSM_DL_GetInfo (const CSSM_GUID_PTR GUID,
                                         CSSM_BOOL CompleteCapabilitiesOnly,
                                         uint32 *NumberOfInfos);

CSSM_RETURN CSSMAPI CSSM_DL_FreeInfo (CSSM_DLINFO_PTR DLInfo,
                                      uint32 NumberOfInfos);

CSSM_NAME_LIST_PTR CSSMAPI CSSM_DL_GetDbNames (CSSM_DL_HANDLE DLHandle);

CSSM_RETURN CSSMAPI CSSM_DL_FreeNameList (CSSM_DL_HANDLE DLHandle,
                                          CSSM_NAME_LIST_PTR NameList);

/* Utility Functions */
CSSM_RETURN CSSMAPI CSSM_VerifyComponents (void);

CSSM_RETURN CSSMAPI CSSM_FreeList (CSSM_LIST_PTR List);

void CSSMAPI CSSM_Free (void *MemPtr, CSSM_HANDLE AddInHandle);

CSSM_API_MEMORY_FUNCS_PTR CSSMAPI CSSM_GetAPIMemoryFunctions (
                                                   CSSM_HANDLE AddInHandle);

/* Error Handling */

#define CSSM_CompareGuids(guid1, guid2)    \
    ( ( (guid1.Data1 == guid2.Data1) &&    \
        (guid1.Data2 == guid2.Data2) &&    \
        (guid1.Data3 == guid2.Data3) &&    \
        (guid1.Data4[0] == guid2.Data4[0]) && \
        (guid1.Data4[1] == guid2.Data4[1]) && \
        (guid1.Data4[2] == guid2.Data4[2]) && \
        (guid1.Data4[3] == guid2.Data4[3]) && \
        (guid1.Data4[4] == guid2.Data4[4]) && \
        (guid1.Data4[5] == guid2.Data4[5]) && \
        (guid1.Data4[6] == guid2.Data4[6]) && \
        (guid1.Data4[7] == guid2.Data4[7]) ) ? CSSM_TRUE : CSSM_FALSE) 

CSSM_RETURN CSSMAPI CSSM_InitError (void);   /* allocate mem for error in that thread */
CSSM_RETURN CSSMAPI CSSM_DestroyError (void);/* free mem for error var in that thread */
CSSM_ERROR_PTR CSSMAPI CSSM_GetError (void);    /* gets the last error */
CSSM_RETURN CSSMAPI CSSM_SetError (CSSM_GUID_PTR guid, uint32  error);/* sets the error var to error */ 
void CSSMAPI CSSM_ClearError(void);        /* clear the value in the error var, 
                i.e., someone returned an error to 
                you, you have handled it and it is 
                no longer a valid error */


#ifdef __cplusplus
}
#endif

#endif
