/*-----------------------------------------------------------------------
 *      File:   CSSMCLI.H
 *
 * Copyright (c) 1995, 1996, 1997 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
/* 
 * WARNING: EXPORT RESTRICTED. 
 * This software listing contains cryptographic methods and technology. 
 * It is export restricted by the Office of Defense Trade Controls, United 
 * States Department of State and cannot be downloaded or otherwise 
 * exported or re-exported (i) into (or to a national or resident of) Cuba, 
 * Iraq, Libya, Yugoslavia, North Korea, Iran, Syria or any other country 
 * to which the US has embargoed goods; or (ii) to anyone on the US 
 * Treasury Department's list of Specially Designated Nationals or the US 
 * Commerce Department's Table of Denial Orders. By downloading or using 
 * this product, you are agreeing to the foregoing and you are representing 
 * and warranting that you are not located in, under the control of, or a 
 * national or resident of any such country or on any such list. 
 */ 

#ifndef _CSSMCLI_H
#define _CSSMCLI_H    

#include "cssmtype.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct cssm_spi_cl_funcs {
    CSSM_DATA_PTR (CSSMAPI *CertSign)  (CSSM_CL_HANDLE CLHandle, 
                                        CSSM_CC_HANDLE CCHandle, 
                                        const CSSM_DATA_PTR SubjectCert, 
                                        const CSSM_DATA_PTR SignerCert, 
                                        const CSSM_FIELD_PTR SignScope, 
                                        uint32 ScopeSize);
    CSSM_DATA_PTR (CSSMAPI *CertUnsign)(CSSM_CL_HANDLE CLHandle, 
                                        CSSM_CC_HANDLE CCHandle, 
                                        const CSSM_DATA_PTR SubjectCert, 
                                        const CSSM_DATA_PTR SignerCert,
                                        const CSSM_FIELD_PTR SignScope,
                                        uint32 ScopeSize);
    CSSM_BOOL     (CSSMAPI *CertVerify)(CSSM_CL_HANDLE CLHandle, 
                                        CSSM_CC_HANDLE CCHandle, 
                                        const CSSM_DATA_PTR SubjectCert, 
                                        const CSSM_DATA_PTR SignerCert, 
                                        const CSSM_FIELD_PTR VerifyScope,
                                        uint32 ScopeSize);
    CSSM_DATA_PTR (CSSMAPI *CertCreate)(CSSM_CL_HANDLE CLHandle, 
                                        const CSSM_FIELD_PTR CertTemplate, 
                                        uint32 NumberOfFields);
    CSSM_FIELD_PTR (CSSMAPI *CertView) (CSSM_CL_HANDLE CLHandle, 
                                        const CSSM_DATA_PTR Cert, 
                                        uint32 *NumberOfFields);
    CSSM_DATA_PTR  (CSSMAPI *CertGetFirstFieldValue) 
                                       (CSSM_CL_HANDLE CLHandle, 
                                        const CSSM_DATA_PTR Cert, 
                                        const CSSM_OID_PTR CertField,
                                        CSSM_HANDLE_PTR ResultsHandle,
                                        uint32 *NumberOfMatchedFields);
    CSSM_DATA_PTR  (CSSMAPI *CertGetNextFieldValue) 
                                       (CSSM_CL_HANDLE CLHandle, 
                                        CSSM_HANDLE ResultsHandle);
    CSSM_RETURN    (CSSMAPI *CertAbortQuery) 
                                       (CSSM_CL_HANDLE CLHandle, 
                                        CSSM_HANDLE ResultsHandle);
    CSSM_KEY_PTR   (CSSMAPI *CertGetKeyInfo) 
                                       (CSSM_CL_HANDLE CLHandle, 
                                        const CSSM_DATA_PTR Cert);
    CSSM_FIELD_PTR (CSSMAPI *CertGetAllFields) 
                                       (CSSM_CL_HANDLE CLHandle,
                                        const CSSM_DATA_PTR Cert,
                                        uint32 *NumberOfFields);
    CSSM_CERTGROUP_PTR (CSSMAPI *CertGroupConstruct) 
                                       (CSSM_CL_HANDLE CLHandle,
                                        CSSM_CERTGROUP_PTR CertGroupFrag,
                                        CSSM_DB_LIST_PTR DBList);
    CSSM_CERTGROUP_PTR (CSSMAPI *CertGroupPrune) 
                                       (CSSM_CL_HANDLE CLHandle,
                                        CSSM_CERTGROUP_PTR OrderedCertGroup,
                                        CSSM_DB_LIST_PTR DBList);
    CSSM_BOOL (CSSMAPI *CertGroupVerify)(CSSM_CL_HANDLE CLHandle,
                                        CSSM_CSP_HANDLE CSPHandle,
                                        CSSM_CERTGROUP_PTR OrderedCertGroup,
                                        const CSSM_FIELD_PTR VerifyScope,
                                        uint32 ScopeSize,
                                        CSSM_DB_LIST_PTR DBList,
                                        CSSM_CERTGROUP_PTR RootCerts);
    CSSM_DATA_PTR (CSSMAPI *CertImport)(CSSM_CL_HANDLE CLHandle, 
                                        CSSM_CERT_TYPE ForeignCertType, 
                                        const CSSM_DATA_PTR ForeignCert); 
    CSSM_DATA_PTR (CSSMAPI *CertExport)(CSSM_CL_HANDLE CLHandle, 
                                        CSSM_CERT_TYPE TargetCertType, 
                                        const CSSM_DATA_PTR NativeCert);
    CSSM_OID_PTR  (CSSMAPI *CertDescribeFormat) 
                                       (CSSM_CL_HANDLE CLHandle,
                                        uint32 *NumberOfFields);
    CSSM_DATA_PTR (CSSMAPI *CrlCreate) (CSSM_CL_HANDLE CLHandle); 
    CSSM_DATA_PTR (CSSMAPI *CrlAddCert)(CSSM_CL_HANDLE CLHandle, 
                                        CSSM_CC_HANDLE CCHandle, 
                                        const CSSM_DATA_PTR Cert, 
                                        const CSSM_DATA_PTR RevokerCert, 
                                        CSSM_REVOKE_REASON RevokeReason, 
                                        const CSSM_DATA_PTR OldCrl);
    CSSM_DATA_PTR (CSSMAPI *CrlRemoveCert) 
                                       (CSSM_CL_HANDLE CLHandle, 
                                        const CSSM_DATA_PTR Cert, 
                                        const CSSM_DATA_PTR OldCrl);
    CSSM_DATA_PTR (CSSMAPI *CrlSign)   (CSSM_CL_HANDLE CLHandle, 
                                        CSSM_CC_HANDLE CCHandle, 
                                        const CSSM_DATA_PTR UnsignedCrl, 
                                        const CSSM_DATA_PTR SignerCert, 
                                        const CSSM_FIELD_PTR SignScope,
                                        uint32 ScopeSize);
    CSSM_BOOL     (CSSMAPI *CrlVerify) (CSSM_CL_HANDLE CLHandle, 
                                        CSSM_CC_HANDLE CCHandle, 
                                        const CSSM_DATA_PTR SubjectCrl, 
                                        const CSSM_DATA_PTR SignerCert,
                                        const CSSM_FIELD_PTR VerifyScope,
                                        uint32 ScopeSize);
    CSSM_BOOL     (CSSMAPI *IsCertInCrl)(CSSM_CL_HANDLE CLHandle, 
                                        const CSSM_DATA_PTR Cert, 
                                        const CSSM_DATA_PTR Crl);
    CSSM_DATA_PTR (CSSMAPI *CrlGetFirstFieldValue) 
                                       (CSSM_CL_HANDLE CLHandle, 
                                        const CSSM_DATA_PTR Crl,
                                        const CSSM_OID_PTR CrlField,
                                        CSSM_HANDLE_PTR ResultsHandle,
                                        uint32 *NumberOfMatchedFields);
    CSSM_DATA_PTR (CSSMAPI *CrlGetNextFieldValue) 
                                       (CSSM_CL_HANDLE CLHandle, 
                                        CSSM_HANDLE ResultsHandle);
    CSSM_RETURN   (CSSMAPI *CrlAbortQuery) 
                                       (CSSM_CL_HANDLE CLHandle, 
                                        CSSM_HANDLE ResultsHandle);
    CSSM_OID_PTR  (CSSMAPI *CrlDescribeFormat) 
                                       (CSSM_CL_HANDLE CLHandle,
                                        uint32 *NumberOfFields);
    CSSM_DATA_PTR (CSSMAPI *PassThrough)(CSSM_CL_HANDLE CLHandle, 
                                        CSSM_CC_HANDLE CCHandle, 
                                        uint32 PassThroughId, 
                                        const CSSM_DATA_PTR InputParams);
    CSSM_RETURN   (CSSMAPI *Initialize)(uint32 VerMajor,
                                        uint32 VerMinor);
    CSSM_RETURN (CSSMAPI *Uninitialize)(void);
} CSSM_SPI_CL_FUNCS, *CSSM_SPI_CL_FUNCS_PTR;

/* Certificate Library SPI */
CSSM_RETURN CSSMAPI CSSM_CL_RegisterServices (
                                        const CSSM_GUID_PTR GUID,
                                        const CSSM_SPI_CL_FUNCS_PTR FunctionTable,
                                        CSSM_SPI_MEMORY_FUNCS_PTR UpcallTable,
                                        void *Reserved);
CSSM_RETURN CSSMAPI CSSM_CL_DeregisterServices (const CSSM_GUID_PTR GUID);

#ifdef __cplusplus
}
#endif

#endif
