/*-----------------------------------------------------------------------
 *      File:   CSSMCSPI.H
 *
 * Copyright (c) 1995, 1996, 1997 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
/* 
 * WARNING: EXPORT RESTRICTED. 
 * This software listing contains cryptographic methods and technology. 
 * It is export restricted by the Office of Defense Trade Controls, United 
 * States Department of State and cannot be downloaded or otherwise 
 * exported or re-exported (i) into (or to a national or resident of) Cuba, 
 * Iraq, Libya, Yugoslavia, North Korea, Iran, Syria or any other country 
 * to which the US has embargoed goods; or (ii) to anyone on the US 
 * Treasury Department's list of Specially Designated Nationals or the US 
 * Commerce Department's Table of Denial Orders. By downloading or using 
 * this product, you are agreeing to the foregoing and you are representing 
 * and warranting that you are not located in, under the control of, or a 
 * national or resident of any such country or on any such list. 
 */ 

#ifndef _CSSMCSPI_H
#define _CSSMCSPI_H    

#include "cssmtype.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct cssm_spi_csp_funcs {
    /* Information Functions */
    CSSM_RETURN (CSSMAPI *QuerySize) (CSSM_CSP_HANDLE CSPHandle,
                                      CSSM_CC_HANDLE CCHandle,
                                      const CSSM_CONTEXT_PTR Context,
                                      uint32 SizeOfInput,
                                      uint32 * ReqSizeOutBlock);
    CSSM_CSPINFO_PTR (CSSMAPI *GetCapabilities) 
                                           (CSSM_CSP_HANDLE CSPHandle,
                                            CSSM_BOOL CompleteCapabilitiesOnly,
                                            uint32 *CSPInfoCount);

    /* Crypto Functions */
    CSSM_RETURN (CSSMAPI *SignData) (CSSM_CSP_HANDLE CSPHandle,
                                     CSSM_CC_HANDLE CCHandle,
                                     const CSSM_CONTEXT_PTR Context,
                                     const CSSM_DATA_PTR DataBufs,
                                     uint32 DataBufCount,
                                     CSSM_DATA_PTR Signature);
    CSSM_RETURN (CSSMAPI *SignDataInit) (CSSM_CSP_HANDLE CSPHandle,
                                         CSSM_CC_HANDLE CCHandle,
                                         const CSSM_CONTEXT_PTR Context);
    CSSM_RETURN (CSSMAPI *SignDataUpdate) (CSSM_CSP_HANDLE CSPHandle,
                                           CSSM_CC_HANDLE CCHandle,
                                           const CSSM_DATA_PTR DataBufs,
                                           uint32 DataBufCount);
    CSSM_RETURN (CSSMAPI *SignDataFinal) (CSSM_CSP_HANDLE CSPHandle,
                                          CSSM_CC_HANDLE CCHandle,
                                          CSSM_DATA_PTR Signature);
    CSSM_BOOL (CSSMAPI *VerifyData) (CSSM_CSP_HANDLE CSPHandle,
                                     CSSM_CC_HANDLE CCHandle,
                                     const CSSM_CONTEXT_PTR Context,
                                     const CSSM_DATA_PTR DataBufs,
                                     uint32 DataBufCount,
                                     const CSSM_DATA_PTR Signature);
    CSSM_RETURN (CSSMAPI *VerifyDataInit) (CSSM_CSP_HANDLE CSPHandle,
                                           CSSM_CC_HANDLE CCHandle,
                                           const CSSM_CONTEXT_PTR Context,
                                           const CSSM_DATA_PTR Signature);
    CSSM_RETURN (CSSMAPI *VerifyDataUpdate) (CSSM_CSP_HANDLE CSPHandle,
                                             CSSM_CC_HANDLE CCHandle,
                                             const CSSM_DATA_PTR DataBufs,
                                             uint32 DataBufCount);
    CSSM_BOOL (CSSMAPI *VerifyDataFinal) (CSSM_CSP_HANDLE CSPHandle,
                                          CSSM_CC_HANDLE CCHandle);
    CSSM_RETURN (CSSMAPI *DigestData) (CSSM_CSP_HANDLE CSPHandle,
                                       CSSM_CC_HANDLE CCHandle,
                                       const CSSM_CONTEXT_PTR Context,
                                       const CSSM_DATA_PTR DataBufs,
                                       uint32 DataBufCount,
                                       CSSM_DATA_PTR Digest);
    CSSM_RETURN (CSSMAPI *DigestDataInit) (CSSM_CSP_HANDLE CSPHandle,
                                           CSSM_CC_HANDLE CCHandle,
                                           const CSSM_CONTEXT_PTR Context);
    CSSM_RETURN (CSSMAPI *DigestDataUpdate) (CSSM_CSP_HANDLE CSPHandle,
                                             CSSM_CC_HANDLE CCHandle,
                                             const CSSM_DATA_PTR DataBufs,
                                             uint32 DataBufCount);
    CSSM_CC_HANDLE (CSSMAPI *DigestDataClone) (CSSM_CSP_HANDLE CSPHandle,
                                               CSSM_CC_HANDLE oldCCHandle,
                                               CSSM_CC_HANDLE newCCHandle);
    CSSM_RETURN (CSSMAPI *DigestDataFinal) (CSSM_CSP_HANDLE CSPHandle,
                                            CSSM_CC_HANDLE CCHandle,
                                            CSSM_DATA_PTR Digest);
    CSSM_RETURN (CSSMAPI *GenerateMac) (CSSM_CSP_HANDLE CSPHandle,
                                        CSSM_CC_HANDLE CCHandle,
                                        const CSSM_CONTEXT_PTR Context,
                                        const CSSM_DATA_PTR DataBufs,
                                        uint32 DataBufCount,
                                        CSSM_DATA_PTR Mac);
    CSSM_RETURN (CSSMAPI *GenerateMacInit) (CSSM_CSP_HANDLE CSPHandle,
                                            CSSM_CC_HANDLE CCHandle,
                                            const CSSM_CONTEXT_PTR Context);
    CSSM_RETURN (CSSMAPI *GenerateMacUpdate) (CSSM_CSP_HANDLE CSPHandle,
                                              CSSM_CC_HANDLE CCHandle,
                                              const CSSM_DATA_PTR DataBufs,
                                              uint32 DataBufCount);
    CSSM_RETURN (CSSMAPI *GenerateMacFinal) (CSSM_CSP_HANDLE CSPHandle,
                                             CSSM_CC_HANDLE CCHandle,
                                             CSSM_DATA_PTR Mac);
    CSSM_RETURN (CSSMAPI *EncryptData) (CSSM_CSP_HANDLE CSPHandle,
                                        CSSM_CC_HANDLE CCHandle,
                                        const CSSM_CONTEXT_PTR Context,
                                        const CSSM_DATA_PTR ClearBufs,
                                        uint32 ClearBufCount,
                                        CSSM_DATA_PTR CipherBufs,
                                        uint32 CipherBufCount,
                                        uint32 *bytesEncrypted,
                                        CSSM_DATA_PTR RemData);
    CSSM_RETURN (CSSMAPI *EncryptDataInit) (CSSM_CSP_HANDLE CSPHandle,
                                            CSSM_CC_HANDLE CCHandle,
                                            const CSSM_CONTEXT_PTR Context);
    CSSM_RETURN (CSSMAPI *EncryptDataUpdate) (CSSM_CSP_HANDLE CSPHandle,
                                              CSSM_CC_HANDLE CCHandle,
                                              const CSSM_DATA_PTR ClearBufs,
                                              uint32 ClearBufCount,
                                              CSSM_DATA_PTR CipherBufs,
                                              uint32 CipherBufCount,
                                              uint32 *bytesEncrypted);
    CSSM_RETURN (CSSMAPI *EncryptDataFinal) (CSSM_CSP_HANDLE CSPHandle,
                                             CSSM_CC_HANDLE CCHandle,
                                             CSSM_DATA_PTR RemData);
    CSSM_RETURN (CSSMAPI *DecryptData) (CSSM_CSP_HANDLE CSPHandle,
                                        CSSM_CC_HANDLE CCHandle,
                                        const CSSM_CONTEXT_PTR Context,
                                        const CSSM_DATA_PTR CipherBufs,
                                        uint32 CipherBufCount,
                                        CSSM_DATA_PTR ClearBufs,
                                        uint32 ClearBufCount,
                                        uint32 *bytesDecrypted,
                                        CSSM_DATA_PTR RemData);
    CSSM_RETURN (CSSMAPI *DecryptDataInit) (CSSM_CSP_HANDLE CSPHandle,
                                            CSSM_CC_HANDLE CCHandle,
                                            const CSSM_CONTEXT_PTR Context);
    CSSM_RETURN (CSSMAPI *DecryptDataUpdate) (CSSM_CSP_HANDLE CSPHandle,
                                              CSSM_CC_HANDLE CCHandle,
                                              const CSSM_DATA_PTR CipherBufs,
                                              uint32 CipherBufCount,
                                              CSSM_DATA_PTR ClearBufs,
                                              uint32 ClearBufCount,
                                              uint32 *bytesDecrypted);
    CSSM_RETURN (CSSMAPI *DecryptDataFinal) (CSSM_CSP_HANDLE CSPHandle,
                                             CSSM_CC_HANDLE CCHandle,
                                             CSSM_DATA_PTR RemData);
    CSSM_RETURN (CSSMAPI *GenerateKey) (CSSM_CSP_HANDLE CSPHandle,
                                        CSSM_CC_HANDLE CCHandle,
                                        const CSSM_CONTEXT_PTR Context,
                                        CSSM_KEY_PTR Key);
    CSSM_RETURN (CSSMAPI *GenerateRandom) (CSSM_CSP_HANDLE CSPHandle,
                                           CSSM_CC_HANDLE CCHandle,
                                           const CSSM_CONTEXT_PTR Context,
                                           CSSM_DATA_PTR RandomNumber);
    CSSM_RETURN (CSSMAPI *GenerateUniqueId) (CSSM_CSP_HANDLE CSPHandle,
                                             CSSM_CC_HANDLE CCHandle,
                                             const CSSM_CONTEXT_PTR Context,
                                             CSSM_DATA_PTR UniqueID);
    CSSM_RETURN (CSSMAPI *KeyExchGenParam) (CSSM_CSP_HANDLE CSPHandle,
                                            CSSM_CC_HANDLE CCHandle,
                                            const CSSM_CONTEXT_PTR Context,
                                            uint32 ParamBits,
                                            CSSM_DATA_PTR Param);
    CSSM_RETURN (CSSMAPI *KeyExchPhase1) (CSSM_CSP_HANDLE CSPHandle,
                                          CSSM_CC_HANDLE CCHandle, 
                                          const CSSM_DATA_PTR Param,
                                          CSSM_DATA_PTR Param1);
    CSSM_RETURN (CSSMAPI *KeyExchPhase2) (CSSM_CSP_HANDLE CSPHandle,
                                          CSSM_CC_HANDLE CCHandle, 
                                          const CSSM_DATA_PTR Param1,
                                          CSSM_KEY_PTR ExchangedKey);
    CSSM_RETURN (CSSMAPI *WrapKey) (CSSM_CSP_HANDLE CSPHandle,
                                    CSSM_CC_HANDLE CCHandle,
                                    const CSSM_CONTEXT_PTR Context,
                                    const CSSM_CRYPTO_DATA_PTR PassPhrase, 
                                    CSSM_KEY_PTR Key,
                                    CSSM_WRAP_KEY_PTR WrappedKey);
    CSSM_RETURN (CSSMAPI *UnwrapKey) (CSSM_CSP_HANDLE CSPHandle, 
                                      CSSM_CC_HANDLE CCHandle,
                                      const CSSM_CONTEXT_PTR Context,
                                      const CSSM_CRYPTO_DATA_PTR NewPassPhrase,
                                      const CSSM_WRAP_KEY_PTR WrappedKey,
                                      CSSM_BOOL StoreKey,
                                      CSSM_KEY_PTR UnwrappedKey);
    CSSM_RETURN (CSSMAPI *DeriveKey) (CSSM_CSP_HANDLE CSPHandle,
                                      CSSM_CC_HANDLE CCHandle,
                                      const CSSM_CONTEXT_PTR Context,
                                      const CSSM_KEY_PTR BaseKey,
                                      CSSM_DATA_PTR Param,
                                      CSSM_BOOL StoreKey,
                                      CSSM_KEY_PTR DerivedKey);

    /* User Login Functions */
    CSSM_RETURN (CSSMAPI *Login) (CSSM_CSP_HANDLE CSPHandle,
                                  const CSSM_CRYPTO_DATA_PTR Password,
                                  const CSSM_DATA_PTR Reserved);
    CSSM_RETURN (CSSMAPI *Logout) (CSSM_CSP_HANDLE CSPHandle);
    CSSM_RETURN (CSSMAPI *ChangeLoginPassword) 
                                      (CSSM_CSP_HANDLE CSPHandle,
                                       const CSSM_CRYPTO_DATA_PTR OldPassword,
                                       const CSSM_CRYPTO_DATA_PTR NewPassword);

    /* Expandability Functions */
    CSSM_RETURN (CSSMAPI *PassThrough) (CSSM_CSP_HANDLE CSPHandle, 
                                        CSSM_CC_HANDLE CCHandle,
                                        const CSSM_CONTEXT_PTR Context,
                                        uint32 PassThroughId,
                                        const CSSM_DATA_PTR InData,
                                        CSSM_DATA_PTR OutData);

    /* Loading, Unloading and Event Notifications */
    CSSM_RETURN (CSSMAPI *Initialize) (uint32 VerMajor,
                                       uint32 VerMinor);
    CSSM_RETURN (CSSMAPI *Uninitialize) (void);
    CSSM_RETURN (CSSMAPI *EventNotify) (CSSM_CSP_HANDLE CSPHandle,
                                        const CSSM_EVENT_TYPE Event,
                                        const uint32 Param);
} CSSM_SPI_CSP_FUNCS, *CSSM_SPI_CSP_FUNCS_PTR;

/* Cryptography SPI */
CSSM_RETURN CSSMAPI CSSM_CSP_RegisterServices 
                                        (const CSSM_GUID_PTR GUID,
                                         const CSSM_SPI_CSP_FUNCS_PTR FunctionTable,
                                         CSSM_SPI_MEMORY_FUNCS_PTR UpcallTable,
                                         void *Reserved);
CSSM_RETURN CSSMAPI CSSM_CSP_DeregisterServices (const CSSM_GUID_PTR GUID);

#ifndef __CSSM_GETHANDLEINFO__
#define __CSSM_GETHANDLEINFO__
CSSM_HANDLEINFO_PTR CSSMAPI CSSM_GetHandleInfo  (CSSM_HANDLE Module);
#endif

#ifdef __cplusplus
}
#endif


#endif
