/*-----------------------------------------------------------------------
 *      File:   CSSMDLI.H
 *
 * Copyright (c) 1995, 1996, 1997 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
/* 
 * WARNING: EXPORT RESTRICTED. 
 * This software listing contains cryptographic methods and technology. 
 * It is export restricted by the Office of Defense Trade Controls, United 
 * States Department of State and cannot be downloaded or otherwise 
 * exported or re-exported (i) into (or to a national or resident of) Cuba, 
 * Iraq, Libya, Yugoslavia, North Korea, Iran, Syria or any other country 
 * to which the US has embargoed goods; or (ii) to anyone on the US 
 * Treasury Department's list of Specially Designated Nationals or the US 
 * Commerce Department's Table of Denial Orders. By downloading or using 
 * this product, you are agreeing to the foregoing and you are representing 
 * and warranting that you are not located in, under the control of, or a 
 * national or resident of any such country or on any such list. 
 */ 

#ifndef _CSSMDLI_H
#define _CSSMDLI_H    

#include "cssmtype.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct cssm_spi_dl_funcs {
    CSSM_DB_HANDLE (CSSMAPI *DbOpen) (CSSM_DL_HANDLE DLHandle,
                                      const char *DbName);

    CSSM_RETURN (CSSMAPI *DbClose) (CSSM_DL_HANDLE DLHandle, 
                                    CSSM_DB_HANDLE DBHandle);

    CSSM_DB_HANDLE (CSSMAPI *DbCreate) (CSSM_DL_HANDLE DLHandle,
                                        CSSM_TP_HANDLE TPHandle, 
                                        const char *DbName, 
                                        CSSM_MODULE_HANDLE_PTR ModuleHandles,
                                        CSSM_DBINFO_PTR DataStoreSchema);

    CSSM_RETURN (CSSMAPI *DbDelete) (CSSM_DL_HANDLE DLHandle, 
                                     const char *DbName);

    CSSM_RETURN (CSSMAPI *DataInsert) (CSSM_DL_HANDLE DLHandle,
                                       CSSM_DB_HANDLE DBHandle,
                                       CSSM_MODULE_HANDLE_PTR  ModuleHandles,
                                       CSSM_DATA_RECORD_TYPE DataRecordType,
                                       CSSM_INDEX_RECORD_PTR IndexRecord,
                                       const CSSM_DATA_PTR DataRecord);

    CSSM_RETURN (CSSMAPI *DataDelete) (CSSM_DL_HANDLE DLHandle,
                                       CSSM_DB_HANDLE DBHandle,
                                       CSSM_MODULE_HANDLE_PTR  ModuleHandles,
                                       CSSM_DATA_RECORD_TYPE DataRecordType,
                                       CSSM_INDEX_RECORD_PTR IndexRecord,
                                       const CSSM_DATA_PTR DataRecord);

    CSSM_DATA_PTR (CSSMAPI *DataGetFirst) (CSSM_DL_HANDLE DLHandle,
                                           CSSM_DB_HANDLE DBHandle, 
                                           CSSM_DATA_RECORD_TYPE DataRecordType,
                                           CSSM_QUERY_TAG QueryTag,
                                           CSSM_QUERY_PTR Query,
                                           CSSM_HANDLE_PTR ResultsHandle,
                                           uint32 *NumberOfMatchedDataRecords);

    CSSM_DATA_PTR (CSSMAPI *DataGetNext) (
                                      CSSM_DL_HANDLE DLHandle,
                                      CSSM_DB_HANDLE DBHandle,
                                      CSSM_HANDLE ResultsHandle, 
                                      CSSM_DATA_RECORD_TYPE_PTR DataRecordType);

    CSSM_RETURN (CSSMAPI *DataAbortQuery) (CSSM_DL_HANDLE DLHandle,
                                           CSSM_DB_HANDLE DBHandle,
                                           CSSM_HANDLE ResultsHandle);


    CSSM_RETURN (CSSMAPI *CertInsert) (CSSM_DL_HANDLE DLHandle,
                                       CSSM_DB_HANDLE DBHandle, 
                                       CSSM_CL_HANDLE CLHandle,
                                       const CSSM_DATA_PTR Cert); 

    CSSM_RETURN (CSSMAPI *CertDelete) (CSSM_DL_HANDLE DLHandle, 
                                       CSSM_DB_HANDLE DBHandle,
                                       CSSM_CL_HANDLE CLHandle,
                                       const CSSM_DATA_PTR Cert);

    CSSM_RETURN (CSSMAPI *CertRevoke) (CSSM_DL_HANDLE DLHandle,
                                       CSSM_DB_HANDLE DBHandle,
                                       const CSSM_DATA_PTR CertToBeRevoked);
    CSSM_DATA_PTR (CSSMAPI *CertGetFirst) (
                              CSSM_DL_HANDLE DLHandle, 
                              CSSM_DB_HANDLE DBHandle, 
                              CSSM_SELECTION_PREDICATE_PTR SelectionPredicate, 
                              uint32 SizeSelectionPredicate,
                              CSSM_DB_CONJUNCTIVE Conjunctive,
                              CSSM_HANDLE_PTR ResultsHandle,
                              uint32 *NumberOfMatchedCerts);

    CSSM_DATA_PTR (CSSMAPI *CertGetNext) (CSSM_DL_HANDLE DLHandle, 
                                          CSSM_DB_HANDLE DBHandle, 
                                          CSSM_HANDLE ResultsHandle);

    CSSM_RETURN (CSSMAPI *CertAbortQuery) (CSSM_DL_HANDLE DLHandle, 
                                           CSSM_DB_HANDLE DBHandle,
                                           CSSM_HANDLE ResultsHandle);

    CSSM_RETURN (CSSMAPI *CrlInsert) (CSSM_DL_HANDLE DLHandle, 
                                      CSSM_DB_HANDLE DBHandle,
                                      CSSM_CL_HANDLE CLHandle,
                                      const CSSM_DATA_PTR Crl); 

    CSSM_RETURN (CSSMAPI *CrlDelete) (CSSM_DL_HANDLE DLHandle, 
                                      CSSM_DB_HANDLE DBHandle, 
                                      CSSM_CL_HANDLE CLHandle,
                                      const CSSM_DATA_PTR Crl);

    CSSM_DATA_PTR (CSSMAPI *CrlGetFirst) (
                              CSSM_DL_HANDLE DLHandle, 
                              CSSM_DB_HANDLE DBHandle, 
                              CSSM_SELECTION_PREDICATE_PTR SelectionPredicate, 
                              uint32 SizeSelectionPredicate,
                              CSSM_DB_CONJUNCTIVE Conjunctive,
                              CSSM_HANDLE_PTR ResultsHandle,
                              uint32 *NumberOfMatchedCrls);

    CSSM_DATA_PTR (CSSMAPI *CrlGetNext) (CSSM_DL_HANDLE DLHandle, 
                                         CSSM_DB_HANDLE DBHandle, 
                                         CSSM_HANDLE ResultsHandle);

    CSSM_RETURN (CSSMAPI *CrlAbortQuery) (CSSM_DL_HANDLE DLHandle, 
                                          CSSM_DB_HANDLE DBHandle,
                                          CSSM_HANDLE ResultsHandle);

    CSSM_RETURN (CSSMAPI *DbImport) (CSSM_DL_HANDLE DLHandle, 
                                     const char *DbDestLogicalName, 
                                     const char *DbSrcFileName);

    CSSM_RETURN (CSSMAPI *DbExport) (CSSM_DL_HANDLE DLHandle, 
                                     const char *DbSrcLogicalName, 
                                     const char *DbDestFileName);
    CSSM_RETURN (CSSMAPI *DbSetInfo) (CSSM_DL_HANDLE DLHandle, 
                                      const char* DbName
                                      CSSM_DBINFO_PTR DbInfo);

    CSSM_DBINFO_PTR (CSSMAPI *DbGetInfo) (CSSM_DL_HANDLE DLHandle, 
                                          const char* DbName);

    CSSM_RETURN (CSSMAPI *FreeDbInfo) (CSSM_DBNFO_PTR DbInfo,
                                       uint32 NumberOfDbInfos);

    CSSM_NAME_LIST_PTR (CSSMAPI *GetDbHandleToName)  (CSSM_DL_HANDLE DLHandle, 
                                                      CSSM_DB_HANDLE DbHandle); 


    CSSM_DATA_PTR (CSSMAPI *PassThrough) (CSSM_DL_HANDLE DLHandle, 
                                          CSSM_DB_HANDLE DBHandle, 
                                          uint32 PassThroughId,
                                          const CSSM_DATA_PTR InputParams); 

    CSSM_NAME_LIST_PTR (CSSMAPI *DLGetDbNames) (CSSM_DL_HANDLE DLHandle);

    CSSM_RETURN (CSSMAPI *DLFreeNameList) (CSSM_DL_HANDLE DLHandle,
                                           CSSM_NAME_LIST_PTR NameList);



    CSSM_RETURN (CSSMAPI *Initialize) (uint32 VerMajor,
                                       uint32 VerMinor);
    CSSM_RETURN (CSSMAPI *Uninitialize) (void);
}CSSM_SPI_DL_FUNCS, *CSSM_SPI_DL_FUNCS_PTR;

/* Database Library SPI */
CSSM_RETURN CSSMAPI CSSM_DL_RegisterServices (
                                      const CSSM_GUID_PTR GUID,
                                      const CSSM_SPI_DL_FUNCS_PTR FunctionTable,
                                      CSSM_SPI_MEMORY_FUNCS_PTR UpcallTable,
                                      void *Reserved);
CSSM_RETURN CSSMAPI CSSM_DL_DeregisterServices (const CSSM_GUID_PTR GUID);

#ifndef __CSSM_GETHANDLEINFO__
#define __CSSM_GETHANDLEINFO__
CSSM_HANDLEINFO_PTR CSSMAPI CSSM_GetHandleInfo (CSSM_HANDLE Module);
#endif

#ifdef __cplusplus
}
#endif

#endif
