/*-----------------------------------------------------------------------
 *      File:   CSSMERR.H
 *
 * Copyright (c) 1995, 1996, 1997 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
/* 
 * WARNING: EXPORT RESTRICTED. 
 * This software listing contains cryptographic methods and technology. 
 * It is export restricted by the Office of Defense Trade Controls, United 
 * States Department of State and cannot be downloaded or otherwise 
 * exported or re-exported (i) into (or to a national or resident of) Cuba, 
 * Iraq, Libya, Yugoslavia, North Korea, Iran, Syria or any other country 
 * to which the US has embargoed goods; or (ii) to anyone on the US 
 * Treasury Department's list of Specially Designated Nationals or the US 
 * Commerce Department's Table of Denial Orders. By downloading or using 
 * this product, you are agreeing to the foregoing and you are representing 
 * and warranting that you are not located in, under the control of, or a 
 * national or resident of any such country or on any such list. 
 */ 

#ifndef _CSSMERR_H
#define _CSSMERR_H


#define CSSM_CSP_BASE_ERROR     ( 1000UL )    /* Defined by CSSM */
#define CSSM_CSP_PRIVATE_ERROR  ( 2000UL )    /* Defined by individual CSPs */
#define CSSM_CSP_END_ERROR      ( 2999UL )
#define CSSM_CL_BASE_ERROR      ( 3000UL )    /* Defined by CSSM */
#define CSSM_CL_PRIVATE_ERROR   ( 4000UL )    /* Definted by indiv CLs */
#define CSSM_CL_END_ERROR       ( 4999UL )
#define CSSM_DL_BASE_ERROR      ( 5000UL )    /* Defined by CSSM */
#define CSSM_DL_PRIVATE_ERROR   ( 6000UL )    /* Defined by indiv DBs */
#define CSSM_DL_END_ERROR       ( 6999UL )
#define CSSM_TP_BASE_ERROR      ( 7000UL )    /* Defined by CSSM */
#define CSSM_TP_PRIVATE_ERROR   ( 8000UL )    /* Defined by indiv TPs */
#define CSSM_TP_END_ERROR       ( 8999UL )

#define CSSM_BASE_ERROR         ( 10000UL )
#define CSSM_END_ERROR          ( 19999UL )

#define CSSM_IsCSPError(err) (((err >= CSSM_CSP_BASE_ERROR) && \
    (err <= CSSM_CSP_END_ERROR)) ? 1 : 0)
#define CSSM_IsCLError(err) (((err >= CSSM_CL_BASE_ERROR) && \
    (err <= CSSM_CL_END_ERROR)) ? 1 : 0)
#define CSSM_IsDLError(err) (((err >= CSSM_DL_BASE_ERROR) && \
    (err <= CSSM_DL_END_ERROR)) ? 1 : 0)
#define CSSM_IsTPError(err) (((err >= CSSM_TP_BASE_ERROR) && \
    (err <= CSSM_TP_END_ERROR)) ? 1 : 0)
#define CSSM_IsCSSMError(err) (((err >= CSSM_BASE_ERROR) && \
    (err <= CSSM_END_ERROR)) ? 1 : 0)
                                    

/* General CSP messages and errors */
#define CSSM_CSP_UNKNOWN_ERROR               ( CSSM_CSP_BASE_ERROR +1L )
#define CSSM_CSP_REGISTER_ERROR              ( CSSM_CSP_BASE_ERROR +2L )
#define CSSM_CSP_VERSION_ERROR               ( CSSM_CSP_BASE_ERROR +3L )
#define CSSM_CSP_CONVERSION_ERROR            ( CSSM_CSP_BASE_ERROR +4L )
#define CSSM_CSP_NO_TOKENINFO                ( CSSM_CSP_BASE_ERROR +5L )
#define CSSM_CSP_INTERNAL_ERROR              ( CSSM_CSP_BASE_ERROR +6L )
#define CSSM_CSP_SERIAL_REQUIRED             ( CSSM_CSP_BASE_ERROR +7L )

#define CSSM_CSP_MEMORY_ERROR                ( CSSM_CSP_BASE_ERROR +10L )
#define CSSM_CSP_NOT_ENOUGH_BUFFER           ( CSSM_CSP_BASE_ERROR +11L )
#define CSSM_CSP_ERR_OUTBUF_LENGTH           ( CSSM_CSP_BASE_ERROR +12L )
#define CSSM_CSP_NO_OUTBUF                   ( CSSM_CSP_BASE_ERROR +13L )
#define CSSM_CSP_ERR_INBUF_LENGTH            ( CSSM_CSP_BASE_ERROR +14L )

#define CSSM_CSP_INVALID_CSP_HANDLE          ( CSSM_CSP_BASE_ERROR +20L )
#define CSSM_CSP_INVALID_POINTER             ( CSSM_CSP_BASE_ERROR +21L )
#define CSSM_CSP_INVALID_CERTIFICATE         ( CSSM_CSP_BASE_ERROR +22L )
#define CSSM_CSP_INVALID_ALGORITHM           ( CSSM_CSP_BASE_ERROR +23L )
#define CSSM_CSP_INVALID_WINDOW_HANDLE       ( CSSM_CSP_BASE_ERROR +24L )
#define CSSM_CSP_INVALID_CALLBACK            ( CSSM_CSP_BASE_ERROR +25L )
#define CSSM_CSP_INVALID_CONTEXT             ( CSSM_CSP_BASE_ERROR +26L )
#define CSSM_CSP_INVALID_CONTEXT_HANDLE      ( CSSM_CSP_BASE_ERROR +27L )
#define CSSM_CSP_INVALID_CONTEXT_POINTER     ( CSSM_CSP_BASE_ERROR +28L )
#define CSSM_CSP_INVALID_DATA_POINTER        ( CSSM_CSP_BASE_ERROR +29L )
#define CSSM_CSP_INVALID_DATA_COUNT          ( CSSM_CSP_BASE_ERROR +30L )
#define CSSM_CSP_INVALID_KEY_LENGTH          ( CSSM_CSP_BASE_ERROR +31L )
#define CSSM_CSP_INVALID_KEY                 ( CSSM_CSP_BASE_ERROR +32L )
#define CSSM_CSP_INVALID_KEY_POINTER         ( CSSM_CSP_BASE_ERROR +33L )
#define CSSM_CSP_INVALID_ALGORITHM_MODE      ( CSSM_CSP_BASE_ERROR +34L )
#define CSSM_CSP_INVALID_PADDING             ( CSSM_CSP_BASE_ERROR +35L )

#define CSSM_CSP_FILE_NOT_EXISTS             ( CSSM_CSP_BASE_ERROR +40L )
#define CSSM_CSP_FILE_NOT_OPEN               ( CSSM_CSP_BASE_ERROR +41L )
#define CSSM_CSP_FILE_OPEN_FAILED            ( CSSM_CSP_BASE_ERROR +42L )
#define CSSM_CSP_FILE_CREATE_FAILED          ( CSSM_CSP_BASE_ERROR +43L )
#define CSSM_CSP_FILE_READ_FAILED            ( CSSM_CSP_BASE_ERROR +44L )
#define CSSM_CSP_FILE_WRITE_FAILED           ( CSSM_CSP_BASE_ERROR +45L )
#define CSSM_CSP_FILE_CLOSE_FAILED           ( CSSM_CSP_BASE_ERROR +46L )
#define CSSM_CSP_FILE_COPY_FAILED            ( CSSM_CSP_BASE_ERROR +47L )
#define CSSM_CSP_FILE_DELETE_FAILED          ( CSSM_CSP_BASE_ERROR +48L )
#define CSSM_CSP_FILE_FORMAT_ERROR           ( CSSM_CSP_BASE_ERROR +49L)

#define CSSM_CSP_PUBKEY_GET_ERROR            ( CSSM_CSP_BASE_ERROR +50L )
#define CSSM_CSP_QUERY_SIZE_FAILED           ( CSSM_CSP_BASE_ERROR +51L )
#define CSSM_CSP_UNKNOWN_ALGORITHM           ( CSSM_CSP_BASE_ERROR +52L )
#define CSSM_CSP_NO_METHOD                   ( CSSM_CSP_BASE_ERROR +53L )
#define CSSM_CSP_VECTOROFBUFS_UNSUPPORTED    ( CSSM_CSP_BASE_ERROR +54L )
#define CSSM_CSP_STAGED_OPERATION_UNSUPPORTED ( CSSM_CSP_BASE_ERROR +55L )
#define CSSM_CSP_GET_APIMEMFUNC_ERROR         ( CSSM_CSP_BASE_ERROR +56L )
#define CSSM_CSP_INPUT_LENGTH_OVERSIZE        ( CSSM_CSP_BASE_ERROR +57L )
#define CSSM_CSP_INPUT_LENGTH_ERROR           ( CSSM_CSP_BASE_ERROR +58L )
#define CSSM_CSP_INPUT_DATA_ERROR             ( CSSM_CSP_BASE_ERROR +59L )

#define CSSM_CSP_PARAM_NO_PARAM              ( CSSM_CSP_BASE_ERROR +70L )
#define CSSM_CSP_PARAM_NO_PASSWORD           ( CSSM_CSP_BASE_ERROR +71L )
#define CSSM_CSP_PARAM_NO_SEED               ( CSSM_CSP_BASE_ERROR +72L )
#define CSSM_CSP_PARAM_NO_KEY                ( CSSM_CSP_BASE_ERROR +73L )
#define CSSM_CSP_PARAM_NO_SALT               ( CSSM_CSP_BASE_ERROR +74L )
#define CSSM_CSP_PARAM_NO_MODULUS            ( CSSM_CSP_BASE_ERROR +75L )
#define CSSM_CSP_PARAM_NO_INPUT_SIZE         ( CSSM_CSP_BASE_ERROR +76L )
#define CSSM_CSP_PARAM_NO_OUTPUT_SIZE        ( CSSM_CSP_BASE_ERROR +77L )
#define CSSM_CSP_PARAM_NO_KEY_LENGTH         ( CSSM_CSP_BASE_ERROR +78L )
#define CSSM_CSP_PARAM_NO_MODE               ( CSSM_CSP_BASE_ERROR +79L )
#define CSSM_CSP_PARAM_NO_DATA               ( CSSM_CSP_BASE_ERROR +80L )
#define CSSM_CSP_PARAM_NO_INIT_VECTOR        ( CSSM_CSP_BASE_ERROR +81L )
#define CSSM_CSP_PARAM_NO_PADDING            ( CSSM_CSP_BASE_ERROR +82L )
#define CSSM_CSP_PARAM_NO_ROUNDS             ( CSSM_CSP_BASE_ERROR +83L )
#define CSSM_CSP_PARAM_NO_RANDOM             ( CSSM_CSP_BASE_ERROR +84L )
#define CSSM_CSP_PARAM_NO_REMAINDATA         ( CSSM_CSP_BASE_ERROR +85L )

/* password error*/
#define CSSM_CSP_PASSWORD_INCORRECT          ( CSSM_CSP_BASE_ERROR +90L )
#define CSSM_CSP_PASSWORD_SAME               ( CSSM_CSP_BASE_ERROR +91L )
#define CSSM_CSP_PASSWORD_LENGTH_ERROR       ( CSSM_CSP_BASE_ERROR +92L )

/* key, key management messages and errors */
#define CSSM_CSP_PRIKEY_LOAD_ERROR           ( CSSM_CSP_BASE_ERROR +100L )
#define CSSM_CSP_PRIKEY_NOT_FOUND            ( CSSM_CSP_BASE_ERROR +101L )
#define CSSM_CSP_PRIKEY_ALREADY_EXIST        ( CSSM_CSP_BASE_ERROR +102L )
#define CSSM_CSP_PRIKEY_GET_ERROR            ( CSSM_CSP_BASE_ERROR +103L )

#define CSSM_CSP_KEY_DUPLICATE               ( CSSM_CSP_BASE_ERROR +110L )
#define CSSM_CSP_KEY_BAD_KEY                 ( CSSM_CSP_BASE_ERROR +111L )
#define CSSM_CSP_KEY_BAD_LENGTH              ( CSSM_CSP_BASE_ERROR +112L )
#define CSSM_CSP_KEY_NO_PARAM                ( CSSM_CSP_BASE_ERROR +113L )
#define CSSM_CSP_KEY_ALGID_NOTMATCH          ( CSSM_CSP_BASE_ERROR +114L )
#define CSSM_CSP_KEY_BLOBDESC_INCORRECT      ( CSSM_CSP_BASE_ERROR +115L )
#define CSSM_CSP_KEY_DELETE_FAILED           ( CSSM_CSP_BASE_ERROR +116L )
#define CSSM_CSP_KEY_USAGE_INCORRECT         ( CSSM_CSP_BASE_ERROR +117L )
                                        
/* Random number generation (RNG) messages and errors */
#define CSSM_CSP_RNG_FAILED                  ( CSSM_CSP_BASE_ERROR +200L )
#define CSSM_CSP_RNG_UNKNOWN_ALGORITHM       ( CSSM_CSP_BASE_ERROR +201L )
#define CSSM_CSP_RNG_NO_METHOD               ( CSSM_CSP_BASE_ERROR +202L )
                                        
/* Unique ID generation (UIDG) messages and errors */
#define CSSM_CSP_UIDG_FAILED                 ( CSSM_CSP_BASE_ERROR +220L )
#define CSSM_CSP_UIDG_UNKNOWN_ALGORITHM      ( CSSM_CSP_BASE_ERROR +221L )
#define CSSM_CSP_UIDG_NO_METHOD              ( CSSM_CSP_BASE_ERROR +222L )

/* Key generation (KGP) messages and errors */
#define CSSM_CSP_KEYGEN_FAILED               ( CSSM_CSP_BASE_ERROR +210L )
#define CSSM_CSP_KEYGEN_UNKNOWN_ALGORITHM    ( CSSM_CSP_BASE_ERROR +211L )
#define CSSM_CSP_KEYGEN_NO_METHOD            ( CSSM_CSP_BASE_ERROR +212L )

/* encryption/decryption */
#define CSSM_CSP_ENC_UNKNOWN_ALGORITHM       ( CSSM_CSP_BASE_ERROR +240L )
#define CSSM_CSP_ENC_NO_METHOD               ( CSSM_CSP_BASE_ERROR +241L )
#define CSSM_CSP_ENC_FAILED                  ( CSSM_CSP_BASE_ERROR +243L )
#define CSSM_CSP_ENC_INIT_FAILED             ( CSSM_CSP_BASE_ERROR +244L )
#define CSSM_CSP_ENC_UPDATE_FAILED           ( CSSM_CSP_BASE_ERROR +245L )
#define CSSM_CSP_ENC_FINAL_FAILED            ( CSSM_CSP_BASE_ERROR +246L )
#define CSSM_CSP_ENC_BAD_IV_LENGTH           ( CSSM_CSP_BASE_ERROR +247L )
#define CSSM_CSP_ENC_BAD_KEY_LENGTH          ( CSSM_CSP_BASE_ERROR +248L )
#define CSSM_CSP_ENC_UNKNOWN_MODE            ( CSSM_CSP_BASE_ERROR +249L )

#define CSSM_CSP_DEC_UNKNOWN_ALGORITHM       ( CSSM_CSP_BASE_ERROR +250L )
#define CSSM_CSP_DEC_NO_METHOD               ( CSSM_CSP_BASE_ERROR +251L )
#define CSSM_CSP_DEC_FAILED                  ( CSSM_CSP_BASE_ERROR +253L )
#define CSSM_CSP_DEC_INIT_FAILED             ( CSSM_CSP_BASE_ERROR +254L )
#define CSSM_CSP_DEC_UPDATE_FAILED           ( CSSM_CSP_BASE_ERROR +255L )
#define CSSM_CSP_DEC_FINAL_FAILED            ( CSSM_CSP_BASE_ERROR +256L )
#define CSSM_CSP_DEC_BAD_IV_LENGTH           ( CSSM_CSP_BASE_ERROR +257L )
#define CSSM_CSP_DEC_BAD_KEY_LENGTH          ( CSSM_CSP_BASE_ERROR +258L )
#define CSSM_CSP_DEC_UNKNOWN_MODE            ( CSSM_CSP_BASE_ERROR +259L )

/* Sign/Verify messages and errors */
#define CSSM_CSP_SIGN_UNKNOWN_ALGORITHM      ( CSSM_CSP_BASE_ERROR +350L )
#define CSSM_CSP_SIGN_NO_METHOD              ( CSSM_CSP_BASE_ERROR +351L )
#define CSSM_CSP_SIGN_FAILED                 ( CSSM_CSP_BASE_ERROR +352L )
#define CSSM_CSP_SIGN_INIT_FAILED            ( CSSM_CSP_BASE_ERROR +353L )
#define CSSM_CSP_SIGN_UPDATE_FAILED          ( CSSM_CSP_BASE_ERROR +354L )
#define CSSM_CSP_SIGN_FINAL_FAILED           ( CSSM_CSP_BASE_ERROR +355L )

#define CSSM_CSP_VERIFY_FAILED               ( CSSM_CSP_BASE_ERROR +360L )
#define CSSM_CSP_VERIFY_INIT_FAILED          ( CSSM_CSP_BASE_ERROR +361L )
#define CSSM_CSP_VERIFY_UPDATE_FAILED        ( CSSM_CSP_BASE_ERROR +362L )
#define CSSM_CSP_VERIFY_FINAL_FAILED         ( CSSM_CSP_BASE_ERROR +363L )
#define CSSM_CSP_VERIFY_UNKNOWN_ALGORITHM    ( CSSM_CSP_BASE_ERROR +365L )
#define CSSM_CSP_VERIFY_NO_METHOD            ( CSSM_CSP_BASE_ERROR +366L )

/* Digest functions errors */
#define CSSM_CSP_DIGEST_UNKNOWN_ALGORITHM    ( CSSM_CSP_BASE_ERROR +380L )
#define CSSM_CSP_DIGEST_NO_METHOD            ( CSSM_CSP_BASE_ERROR +382L )
#define CSSM_CSP_DIGEST_FAILED               ( CSSM_CSP_BASE_ERROR +383L )
#define CSSM_CSP_DIGEST_INIT_FAILED          ( CSSM_CSP_BASE_ERROR +384L )
#define CSSM_CSP_DIGEST_UPDATE_FAILED        ( CSSM_CSP_BASE_ERROR +385L )
#define CSSM_CSP_DIGEST_CLONE_FAILED         ( CSSM_CSP_BASE_ERROR +386L )
#define CSSM_CSP_DIGEST_FINAL_FAILED         ( CSSM_CSP_BASE_ERROR +387L )
                                        
/* Mac functions errors */
#define CSSM_CSP_MAC_UNKNOWN_ALGORITHM       ( CSSM_CSP_BASE_ERROR +390L )
#define CSSM_CSP_MAC_NO_METHOD               ( CSSM_CSP_BASE_ERROR +392L )
#define CSSM_CSP_MAC_FAILED                  ( CSSM_CSP_BASE_ERROR +393L )
#define CSSM_CSP_MAC_INIT_FAILED             ( CSSM_CSP_BASE_ERROR +394L )
#define CSSM_CSP_MAC_UPDATE_FAILED           ( CSSM_CSP_BASE_ERROR +395L )
#define CSSM_CSP_MAC_CLONE_FAILED            ( CSSM_CSP_BASE_ERROR +396L )
#define CSSM_CSP_MAC_FINAL_FAILED            ( CSSM_CSP_BASE_ERROR +397L )

/* Key exchange errors */
#define CSSM_CSP_KEYEXCH_GENPARAM_FAILED     ( CSSM_CSP_BASE_ERROR +410L )
#define CSSM_CSP_KEYEXCH_PHASE1_FAILED       ( CSSM_CSP_BASE_ERROR +411L )
#define CSSM_CSP_KEYEXCH_PHASE2_FAILED       ( CSSM_CSP_BASE_ERROR +412L )
#define CSSM_CSP_KEYEXCH_UNKNOWN_ALGORITHM   ( CSSM_CSP_BASE_ERROR +413L )
#define CSSM_CSP_KEYEXCH_NO_METHOD           ( CSSM_CSP_BASE_ERROR +414L )

/* pass through custom error*/
#define CSSM_CSP_INVALID_PASSTHROUGH_ID      ( CSSM_CSP_BASE_ERROR +420L )
#define CSSM_CSP_INVALID_PASSTHROUGH_PARAMS  ( CSSM_CSP_BASE_ERROR +421L )

/* wrap/unwrap error*/
#define CSSM_CSP_WRAP_UNKNOWN_ALGORITHM      ( CSSM_CSP_BASE_ERROR +450L )
#define CSSM_CSP_WRAP_NO_METHOD              ( CSSM_CSP_BASE_ERROR +451L )
#define CSSM_CSP_WRAP_FAILED                 ( CSSM_CSP_BASE_ERROR +452L )
#define CSSM_CSP_UNWRAP_UNKNOWN_ALGORITHM    ( CSSM_CSP_BASE_ERROR +456L )
#define CSSM_CSP_UNWRAP_NO_METHOD            ( CSSM_CSP_BASE_ERROR +457L )
#define CSSM_CSP_UNWRAP_FAILED               ( CSSM_CSP_BASE_ERROR +458L )

/* Porting Library Error Codes */

/* Memory allocation, pointers, strings */
#define CSSM_MALLOC_FAILED                   ( CSSM_BASE_ERROR + 1 )
#define CSSM_CALLOC_FAILED                   ( CSSM_BASE_ERROR + 2 )
#define CSSM_REALLOC_FAILED                  ( CSSM_BASE_ERROR + 3 )

/* File I/O */
#define CSSM_FWRITE_FAILED                   ( CSSM_BASE_ERROR + 10 )
#define CSSM_FREAD_FAILED                    ( CSSM_BASE_ERROR + 11 )
#define CSSM_CANT_FSEEK                      ( CSSM_BASE_ERROR + 12 )
#define CSSM_INVALID_FILE_PTR                ( CSSM_BASE_ERROR + 13 )
#define CSSM_END_OF_FILE                     ( CSSM_BASE_ERROR + 14 )

/* Misc Errors */
#define CSSM_CANT_GET_USER_NAME              ( CSSM_BASE_ERROR + 20 )
#define CSSM_GETCWD_FAILED                   ( CSSM_BASE_ERROR + 21 )
#define CSSM_ENV_VAR_NOT_FOUND               ( CSSM_BASE_ERROR + 22 )

/* Dynamic Library */
#define CSSM_FREE_LIBRARY_FAILED             ( CSSM_BASE_ERROR + 30 )
#define CSSM_LOAD_LIBRARY_FAILED             ( CSSM_BASE_ERROR + 31 )
#define CSSM_CANT_GET_PROC_ADDR              ( CSSM_BASE_ERROR + 32 )
#define CSSM_CANT_GET_MODULE_HANDLE          ( CSSM_BASE_ERROR + 33 )
#define CSSM_CANT_GET_MODULE_FILE_NAME       ( CSSM_BASE_ERROR + 34 )
#define CSSM_INVALID_LIB_HANDLE              ( CSSM_BASE_ERROR + 35 )
#define CSSM_BAD_MODULE_HANDLE               ( CSSM_BASE_ERROR + 36 )

/* Registry errors */
#define CSSM_CANT_CREATE_KEY                 ( CSSM_BASE_ERROR + 40 )
#define CSSM_CANT_SET_VALUE                  ( CSSM_BASE_ERROR + 41 )
#define CSSM_CANT_GET_VALUE                  ( CSSM_BASE_ERROR + 42 )
#define CSSM_CANT_DELETE_SECTION             ( CSSM_BASE_ERROR + 43 )
#define CSSM_CANT_DELETE_KEY                 ( CSSM_BASE_ERROR + 44 )
#define CSSM_CANT_ENUM_KEY                   ( CSSM_BASE_ERROR + 45 )
#define CSSM_CANT_OPEN_KEY                   ( CSSM_BASE_ERROR + 46 )
#define CSSM_CANT_QUERY_KEY                  ( CSSM_BASE_ERROR + 47 )

/* Mutex/Synchronization Errors */
#define CSSM_CANT_CREATE_MUTEX               ( CSSM_BASE_ERROR + 50 )
#define CSSM_LOCK_MUTEX_FAILED               ( CSSM_BASE_ERROR + 51 )
#define CSSM_TRYLOCK_MUTEX_FAILED            ( CSSM_BASE_ERROR + 52 )
#define CSSM_UNLOCK_MUTEX_FAILED             ( CSSM_BASE_ERROR + 53 )
#define CSSM_CANT_CLOSE_MUTEX                ( CSSM_BASE_ERROR + 54 )
#define CSSM_INVALID_MUTEX_PTR               ( CSSM_BASE_ERROR + 55 )

/* Shared Memory File Errors */
#define CSSM_CANT_CREATE_SHARED_MEMORY_FILE  ( CSSM_BASE_ERROR + 60 )
#define CSSM_CANT_OPEN_SHARED_MEMORY_FILE    ( CSSM_BASE_ERROR + 61 )
#define CSSM_CANT_MAP_SHARED_MEMORY_FILE     ( CSSM_BASE_ERROR + 62 )
#define CSSM_CANT_UNMAP_SHARED_MEMORY_FILE   ( CSSM_BASE_ERROR + 63 )
#define CSSM_CANT_FLUSH_SHARED_MEMORY_FILE   ( CSSM_BASE_ERROR + 64 )
#define CSSM_CANT_CLOSE_SHARED_MEMORY_FILE   ( CSSM_BASE_ERROR + 65 )
#define CSSM_INVALID_PERMS                   ( CSSM_BASE_ERROR + 66 )
#define CSSM_BAD_FILE_HANDLE                 ( CSSM_BASE_ERROR + 67 )
#define CSSM_BAD_FILE_ADDR                   ( CSSM_BASE_ERROR + 68 )


/* General */
#define CSSM_BAD_PTR_PASSED                  ( CSSM_BASE_ERROR + 100 )

/* CSSM API Errors */

#define CSSM_INVALID_POINTER                 ( CSSM_BASE_ERROR + 301L )
#define CSSM_EXPIRED                         ( CSSM_BASE_ERROR + 302L )
#define CSSM_MEMORY_ERROR                    ( CSSM_BASE_ERROR + 303L )
#define CSSM_INVALID_ATTRIBUTE               ( CSSM_BASE_ERROR + 304L )
#define CSSM_NOT_INITIALIZE                  ( CSSM_BASE_ERROR + 305L )
#define CSSM_INSTALL_FAIL                    ( CSSM_BASE_ERROR + 306L )
#define CSSM_REGISTRY_ERROR                  ( CSSM_BASE_ERROR + 307L )
#define CSSM_INVALID_CONTEXT_HANDLE          ( CSSM_BASE_ERROR + 308L )
#define CSSM_INVALID_CSP_HANDLE              ( CSSM_BASE_ERROR + 309L )
#define CSSM_INVALID_TP_HANDLE               ( CSSM_BASE_ERROR + 310L )
#define CSSM_INVALID_CL_HANDLE               ( CSSM_BASE_ERROR + 311L )
#define CSSM_INVALID_DL_HANDLE               ( CSSM_BASE_ERROR + 312L )
#define CSSM_INCOMPATIBLE_VERSION            ( CSSM_BASE_ERROR + 313L )
#define CSSM_ATTACH_FAIL                     ( CSSM_BASE_ERROR + 314L )
#define CSSM_NO_ADDIN                        ( CSSM_BASE_ERROR + 315L )
#define CSSM_FUNCTION_NOT_IMPLEMENTED        ( CSSM_BASE_ERROR + 316L )


/* CSSM errors, new */

#define CSSM_INVALID_ADDIN_HANDLE            ( CSSM_BASE_ERROR + 500L )
/* internal error for when a plug-in handle of any type is passed
 * to the plug-in manager.
 */
#define CSSM_INVALID_GUID                    ( CSSM_BASE_ERROR + 501L )
/* ill-formed or unmatchable GUID */


/* Certificate Library Errors */
#define CSSM_CL_UNKNOWN_FORMAT               ( CSSM_CL_BASE_ERROR + 1 )
#define CSSM_CL_UNKNOWN_TAG                  ( CSSM_CL_BASE_ERROR + 2 )
#define CSSM_CL_INVALID_CONTEXT              ( CSSM_CL_BASE_ERROR + 3 )
#define CSSM_CL_INVALID_CL_HANDLE            ( CSSM_CL_BASE_ERROR + 4 )
#define CSSM_CL_INVALID_CC_HANDLE            ( CSSM_CL_BASE_ERROR + 5 )
#define CSSM_CL_INVALID_CERT_POINTER         ( CSSM_CL_BASE_ERROR + 6 )
#define CSSM_CL_INVALID_FIELD_POINTER        ( CSSM_CL_BASE_ERROR + 7 )
#define CSSM_CL_INVALID_TEMPLATE             ( CSSM_CL_BASE_ERROR + 8 )
#define CSSM_CL_INVALID_DATA_POINTER         ( CSSM_CL_BASE_ERROR + 9 )
#define CSSM_CL_INVALID_SCOPE                ( CSSM_CL_BASE_ERROR + 10 )
#define CSSM_CL_CERT_CREATE_FAIL             ( CSSM_CL_BASE_ERROR + 12 )
#define CSSM_CL_CERT_VIEW_FAIL               ( CSSM_CL_BASE_ERROR + 13 )
#define CSSM_CL_CERT_GET_FIELD_VALUE_FAIL    ( CSSM_CL_BASE_ERROR + 14 )
#define CSSM_CL_CERT_GET_KEY_INFO_FAIL       ( CSSM_CL_BASE_ERROR + 15 )
#define CSSM_CL_CERT_IMPORT_FAIL             ( CSSM_CL_BASE_ERROR + 16 )
#define CSSM_CL_CERT_EXPORT_FAIL             ( CSSM_CL_BASE_ERROR + 17 )
#define CSSM_CL_PASS_THROUGH_FAIL            ( CSSM_CL_BASE_ERROR + 18 )
#define CSSM_CL_CERT_DESCRIBE_FORMAT_FAIL    ( CSSM_CL_BASE_ERROR + 19 )
#define CSSM_CL_UNSUPPORTED_OPERATION        ( CSSM_CL_BASE_ERROR + 20 )
#define CSSM_CL_MEMORY_ERROR                 ( CSSM_CL_BASE_ERROR + 21 )
#define CSSM_CL_CERT_SIGN_FAIL               ( CSSM_CL_BASE_ERROR + 22 )
#define CSSM_CL_CERT_UNSIGN_FAIL             ( CSSM_CL_BASE_ERROR + 23 )
#define CSSM_CL_CERT_VERIFY_FAIL             ( CSSM_CL_BASE_ERROR + 24 )
#define CSSM_CL_RESULTS_HANDLE               ( CSSM_CL_BASE_ERROR + 25 )
#define CSSM_CL_INVALID_SIGNER_CERTIFICATE   ( CSSM_CL_BASE_ERROR + 26 )
#define CSSM_CL_NO_FIELD_VALUES              ( CSSM_CL_BASE_ERROR + 27 )
#define CSSM_CL_INVALID_CRL_PTR              ( CSSM_CL_BASE_ERROR + 28 )
#define CSSM_CL_CERT_ABORT_QUERY_FAIL        ( CSSM_CL_BASE_ERROR + 29 )
#define CSSM_CL_CRL_CREATE_FAIL              ( CSSM_CL_BASE_ERROR + 30 )
#define CSSM_CL_CRL_ADD_CERT_FAIL            ( CSSM_CL_BASE_ERROR + 32 )
#define CSSM_CL_CRL_REMOVE_CERT_FAIL         ( CSSM_CL_BASE_ERROR + 33 )
#define CSSM_CL_CRL_SIGN_FAIL                ( CSSM_CL_BASE_ERROR + 34 )
#define CSSM_CL_CRL_VERIFY_FAIL              ( CSSM_CL_BASE_ERROR + 35 )
#define CSSM_CL_CRL_GET_FIELD_VALUE_FAIL     ( CSSM_CL_BASE_ERROR + 36 )
#define CSSM_CL_CRL_ABORT_QUERY_FAIL         ( CSSM_CL_BASE_ERROR + 37 )
#define CSSM_CL_CRL_DESCRIBE_FORMAT_FAIL     ( CSSM_CL_BASE_ERROR + 38 )
#define CSSM_CL_INVALID_POINTER              ( CSSM_CL_BASE_ERROR + 39 )
#define CSSM_CL_INVALID_DATA                 ( CSSM_CL_BASE_ERROR + 40 )
#define CSSM_CL_INITIALIZE_FAIL              ( CSSM_CL_BASE_ERROR + 41 )

#define CSSM_CL_INVALID_REVOKER_CERT_PTR     ( CSSM_CL_BASE_ERROR + 101 )
#define CSSM_CL_NO_REVOKED_CERTS_IN_CRL      ( CSSM_CL_BASE_ERROR + 102 )
#define CSSM_CL_CERT_NOT_FOUND_IN_CRL        ( CSSM_CL_BASE_ERROR + 103 )
#define CSSM_CL_CRL_SIGNSCOPE_NOT_SUPPORTED  ( CSSM_CL_BASE_ERROR + 104 )
#define CSSM_CL_CRL_VERIFYSCOPE_NOT_SUPPORTED (CSSM_CL_BASE_ERROR + 105 )
#define CSSM_CL_CRL_NOT_SIGNEDBY_SIGNER      ( CSSM_CL_BASE_ERROR + 106 )
#define CSSM_CL_CRL_NO_FIELD_OID             ( CSSM_CL_BASE_ERROR + 107 )

/* CURRENTLY UNUSED by the Intel CL As OF 10/2/96 */
#define CSSM_CL_SIG_NOT_IN_CERT              ( CSSM_CL_BASE_ERROR + 100 )
#define CSSM_CL_INVALID_REVOKED_CERT_PTR     ( CSSM_CL_BASE_ERROR + 108 )
#define CSSM_CL_INVALID_INPUT_PTR            ( CSSM_CL_BASE_ERROR + 109 )


/* Trust Policy Errors */
#define CSSM_TP_NOT_LOADED                   ( CSSM_TP_BASE_ERROR + 1 )
#define CSSM_TP_INVALID_TP_HANDLE            ( CSSM_TP_BASE_ERROR + 2 )
#define CSSM_TP_INVALID_CL_HANDLE            ( CSSM_TP_BASE_ERROR + 3 )
#define CSSM_TP_INVALID_DL_HANDLE            ( CSSM_TP_BASE_ERROR + 4 )
#define CSSM_TP_INVALID_DB_HANDLE            ( CSSM_TP_BASE_ERROR + 5 )
#define CSSM_TP_INVALID_CC_HANDLE            ( CSSM_TP_BASE_ERROR + 6 )
#define CSSM_TP_INVALID_CERTIFICATE          ( CSSM_TP_BASE_ERROR + 7 )
#define CSSM_TP_NOT_SIGNER                   ( CSSM_TP_BASE_ERROR + 8 )
#define CSSM_TP_NOT_TRUSTED                  ( CSSM_TP_BASE_ERROR + 9 )
#define CSSM_TP_CERT_VERIFY_FAIL             ( CSSM_TP_BASE_ERROR + 10 )
#define CSSM_TP_CERTIFICATE_CANT_OPERATE     ( CSSM_TP_BASE_ERROR + 11 )
#define CSSM_TP_MEMORY_ERROR                 ( CSSM_TP_BASE_ERROR + 12 )
#define CSSM_TP_CERT_SIGN_FAIL               ( CSSM_TP_BASE_ERROR + 13 )
#define CSSM_TP_INVALID_CRL                  ( CSSM_TP_BASE_ERROR + 14 )
#define CSSM_TP_CERT_REVOKE_FAIL             ( CSSM_TP_BASE_ERROR + 15 )
#define CSSM_TP_CRL_VERIFY_FAIL              ( CSSM_TP_BASE_ERROR + 16 )
#define CSSM_TP_CRL_SIGN_FAIL                ( CSSM_TP_BASE_ERROR + 17 )
#define CSSM_TP_APPLY_CRL_TO_DB_FAIL         ( CSSM_TP_BASE_ERROR + 18 )
#define CSSM_TP_INVALID_GUID                 ( CSSM_TP_BASE_ERROR + 19 )
#define CSSM_TP_UNISTALL_FAIL                ( CSSM_TP_BASE_ERROR + 20 )
#define CSSM_TP_INCOMPATIBLE_VERSION         ( CSSM_TP_BASE_ERROR + 21 )
#define CSSM_TP_INVALID_ACTION               ( CSSM_TP_BASE_ERROR + 22 )
#define CSSM_TP_VERIFY_ACTION_FAIL           ( CSSM_TP_BASE_ERROR + 23 )
#define CSSM_TP_INVALID_DATA_POINTER         ( CSSM_TP_BASE_ERROR + 24 )
#define CSSM_TP_INVALID_ID                   ( CSSM_TP_BASE_ERROR + 25 )
#define CSSM_TP_PASS_THROUGH_FAIL            ( CSSM_TP_BASE_ERROR + 26 )

/* Database Errors */
#define CSSM_DL_NOT_LOADED                   ( CSSM_DL_BASE_ERROR + 1 )
#define CSSM_DL_INVALID_DL_HANDLE            ( CSSM_DL_BASE_ERROR + 2 ) 
#define CSSM_DL_DATASTORE_NOT_EXISTS         ( CSSM_DL_BASE_ERROR + 3 ) 
#define CSSM_DL_MEMORY_ERROR                 ( CSSM_DL_BASE_ERROR + 4 ) 
#define CSSM_DL_DB_OPEN_FAIL                 ( CSSM_DL_BASE_ERROR + 5 ) 
#define CSSM_DL_INVALID_DB_HANDLE            ( CSSM_DL_BASE_ERROR + 6 ) 
#define CSSM_DL_DB_CLOSE_FAIL                ( CSSM_DL_BASE_ERROR + 7 ) 
#define CSSM_DL_DB_CREATE_FAIL               ( CSSM_DL_BASE_ERROR + 8 ) 
#define CSSM_DL_DB_DELETE_FAIL               ( CSSM_DL_BASE_ERROR + 9 ) 
#define CSSM_DL_INVALID_PTR                  ( CSSM_DL_BASE_ERROR + 10 )
#define CSSM_DL_DB_IMPORT_FAIL               ( CSSM_DL_BASE_ERROR + 11 )
#define CSSM_DL_DB_EXPORT_FAIL               ( CSSM_DL_BASE_ERROR + 12 )
#define CSSM_DL_INVALID_CERTIFICATE_PTR      ( CSSM_DL_BASE_ERROR + 13 )
#define CSSM_DL_CERT_INSERT_FAIL             ( CSSM_DL_BASE_ERROR + 14 )
#define CSSM_DL_CERTIFICATE_NOT_IN_DB        ( CSSM_DL_BASE_ERROR + 15 )
#define CSSM_DL_CERT_DELETE_FAIL             ( CSSM_DL_BASE_ERROR + 16 )
#define CSSM_DL_CERT_REVOKE_FAIL             ( CSSM_DL_BASE_ERROR + 17 )
#define CSSM_DL_INVALID_SELECTION_PTR        ( CSSM_DL_BASE_ERROR + 18 )
#define CSSM_DL_NO_CERTIFICATE_FOUND         ( CSSM_DL_BASE_ERROR + 19 )
#define CSSM_DL_CERT_GETFIRST_FAIL           ( CSSM_DL_BASE_ERROR + 20 )
#define CSSM_DL_NO_MORE_CERTS                ( CSSM_DL_BASE_ERROR + 21 )
#define CSSM_DL_CERT_GET_NEXT_FAIL           ( CSSM_DL_BASE_ERROR + 22 )
#define CSSM_DL_CERT_ABORT_QUERY_FAIL        ( CSSM_DL_BASE_ERROR + 23 )
#define CSSM_DL_INVALID_CRL_PTR              ( CSSM_DL_BASE_ERROR + 24 )
#define CSSM_DL_CRL_INSERT_FAIL              ( CSSM_DL_BASE_ERROR + 25 )
#define CSSM_DL_CRL_NOT_IN_DB                ( CSSM_DL_BASE_ERROR + 26 )
#define CSSM_DL_CRL_DELETE_FAIL              ( CSSM_DL_BASE_ERROR + 27 )
#define CSSM_DL_NO_CRL_FOUND                 ( CSSM_DL_BASE_ERROR + 28 )
#define CSSM_DL_CRL_GET_FIRST_FAIL           ( CSSM_DL_BASE_ERROR + 29 )
#define CSSM_DL_NO_MORE_CRLS                 ( CSSM_DL_BASE_ERROR + 30 )
#define CSSM_DL_CRL_GET_NEXT_FAIL            ( CSSM_DL_BASE_ERROR + 31 )
#define CSSM_DL_CRL_ABORT_QUERY_FAIL         ( CSSM_DL_BASE_ERROR + 32 )
#define CSSM_DL_GET_DB_NAMES_FAIL            ( CSSM_DL_BASE_ERROR + 33 )
#define CSSM_DL_INVALID_PASSTHROUGH_ID       ( CSSM_DL_BASE_ERROR + 34 )
#define CSSM_DL_PASS_THROUGH_FAIL            ( CSSM_DL_BASE_ERROR + 35 )
#define CSSM_DL_INVALID_POINTER              ( CSSM_DL_BASE_ERROR + 36 )
#define CSSM_DL_NO_DATASOURCES               ( CSSM_DL_BASE_ERROR + 37 )
#define CSSM_DL_INCOMPATIBLE_VERSION         ( CSSM_DL_BASE_ERROR + 38 )
#define CSSM_DL_INVALID_FIELD_INFO             ( CSSM_DL_BASE_ERROR + 39 )
/* CURRENTLY UNUSED by the Intel DL As OF 10/2/96 */
#define CSSM_DL_INVALID_DATA_POINTER         ( CSSM_DL_BASE_ERROR + 100 )
#define CSSM_DL_INVALID_DLINFO_POINTER       ( CSSM_DL_BASE_ERROR + 101 )
#define CSSM_DL_INSTALL_FAIL                 ( CSSM_DL_BASE_ERROR + 102 )
#define CSSM_DL_INVALID_ GUID                ( CSSM_DL_BASE_ERROR + 103 )
#define CSSM_DL_UNINSTALL_FAIL               ( CSSM_DL_BASE_ERROR + 104 )
#define CSSM_DL_LIST_MODULES_FAIL            ( CSSM_DL_BASE_ERROR + 105 )
#define CSSM_DL_ATTACH_FAIL                  ( CSSM_DL_BASE_ERROR + 107 )
#define CSSM_DL_DETACH_FAIL                  ( CSSM_DL_BASE_ERROR + 108 )
#define CSSM_DL_GET_INFO_FAIL                ( CSSM_DL_BASE_ERROR + 109 )
#define CSSM_DL_FREE_INFO_FAIL               ( CSSM_DL_BASE_ERROR + 110 )
#define CSSM_DL_INVALID_DLINFO_PTR           ( CSSM_DL_BASE_ERROR + 111 )
#define CSSM_DL_INVALID_CL_HANDLE            ( CSSM_DL_BASE_ERROR + 112 )
/*#define CSSM_DL_INVALID_CERTIFICATE_PTR    ( CSSM_DL_BASE_ERROR + 113 ) *//* redundant with above */
#define CSSM_DL_INVALID_CRL                  ( CSSM_DL_BASE_ERROR + 114 )
#define CSSM_DL_INVALID_CRL_POINTER          ( CSSM_DL_BASE_ERROR + 115 )

/* Do not edit the following string. It should be expanded at check-in by version control. */
static const char sccs_id_INC_CSSMERR_H[] = { "@(#) $Archive: /cdsa/r1_2/inc/cssmerr.h $, $Revision: 1 $, $Modtime: 3/07/97 10:56a $" }; 

#endif /* _CSSMERR_H */
