/*-----------------------------------------------------------------------
 *      File:   CSSMTPI.H
 *
 * Copyright (c) 1995, 1996, 1997 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
/* 
 * WARNING: EXPORT RESTRICTED. 
 * This software listing contains cryptographic methods and technology. 
 * It is export restricted by the Office of Defense Trade Controls, United 
 * States Department of State and cannot be downloaded or otherwise 
 * exported or re-exported (i) into (or to a national or resident of) Cuba, 
 * Iraq, Libya, Yugoslavia, North Korea, Iran, Syria or any other country 
 * to which the US has embargoed goods; or (ii) to anyone on the US 
 * Treasury Department's list of Specially Designated Nationals or the US 
 * Commerce Department's Table of Denial Orders. By downloading or using 
 * this product, you are agreeing to the foregoing and you are representing 
 * and warranting that you are not located in, under the control of, or a 
 * national or resident of any such country or on any such list. 
 */ 

#ifndef _CSSMTPI_H
#define _CSSMTPI_H    

#include "cssmtype.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct cssm_spi_tp_funcs {
    CSSM_BOOL (CSSMAPI *CertVerify) (CSSM_TP_HANDLE TPHandle, 
                                     CSSM_CL_HANDLE CLHandle, 
                                     CSSM_DL_HANDLE DLHandle, 
                                     CSSM_DB_HANDLE DBHandle, 
                                     CSSM_CC_HANDLE CCHandle, 
                                     const CSSM_DATA_PTR SubjectCert, 
                                     const CSSM_DATA_PTR SignerCert, 
                                     const CSSM_FIELD_PTR VerifyScope,
                                     uint32 ScopeSize);
    CSSM_DATA_PTR (CSSMAPI *CertSign) (CSSM_TP_HANDLE TPHandle, 
                                       CSSM_CL_HANDLE CLHandle, 
                                       CSSM_DL_HANDLE DLHandle, 
                                       CSSM_DB_HANDLE DBHandle, 
                                       CSSM_CC_HANDLE CCHandle, 
                                       const CSSM_DATA_PTR SubjectCert, 
                                       const CSSM_DATA_PTR SignerCert, 
                                       CSSM_FIELD_PTR SignScope, 
                                       uint32 ScopeSize);
    CSSM_DATA_PTR (CSSMAPI *CertRevoke) (CSSM_TP_HANDLE TPHandle, 
                                         CSSM_CL_HANDLE CLHandle, 
                                         CSSM_DL_HANDLE DLHandle, 
                                         CSSM_DB_HANDLE DBHandle, 
                                         CSSM_CC_HANDLE CCHandle,  
                                         const CSSM_DATA_PTR OldCrl, 
                                         const CSSM_DATA_PTR SubjectCert, 
                                         const CSSM_DATA_PTR RevokerCert, 
                                         CSSM_REVOKE_REASON Reason);
    CSSM_BOOL (CSSMAPI *CrlVerify) (CSSM_TP_HANDLE TPHandle, 
                                    CSSM_CL_HANDLE CLHandle, 
                                    CSSM_DL_HANDLE DLHandle, 
                                    CSSM_DB_HANDLE DBHandle, 
                                    CSSM_CC_HANDLE CCHandle,  
                                    const CSSM_DATA_PTR SubjectCrl,
                                    const CSSM_DATA_PTR SignerCert, 
                                    const CSSM_FIELD_PTR VerifyScope,
                                    uint32 ScopeSize);
    CSSM_DATA_PTR (CSSMAPI *CrlSign) (CSSM_TP_HANDLE TPHandle, 
                                      CSSM_CL_HANDLE CLHandle, 
                                      CSSM_DL_HANDLE DLHandle, 
                                      CSSM_DB_HANDLE DBHandle, 
                                      CSSM_CC_HANDLE CCHandle,  
                                      const CSSM_DATA_PTR SubjectCrl, 
                                      const CSSM_DATA_PTR SignerCert, 
                                      const CSSM_FIELD_PTR SignScope,
                                      uint32 ScopeSize);
    CSSM_RETURN (CSSMAPI *ApplyCrlToDb) (CSSM_TP_HANDLE TPHandle, 
                                         CSSM_CL_HANDLE CLHandle, 
                                         CSSM_DL_HANDLE DLHandle, 
                                         CSSM_DB_HANDLE DBHandle, 
                                         const CSSM_DATA_PTR Crl);
    CSSM_RETURN (CSSMAPI *VerifyAction) (CSSM_TP_HANDLE TPHandle, 
                                         CSSM_CL_HANDLE CLHandle, 
                                         CSSM_DL_HANDLE DLHandle, 
                                         CSSM_DB_HANDLE DBHandle, 
                                         CSSM_CC_HANDLE CCHandle,  
                                         CSSM_TP_ACTION Action, 
                                         const CSSM_DATA_PTR Data, 
                                         const CSSM_DATA_PTR Cert);
    CSSM_DATA_PTR (CSSMAPI *PassThrough) (CSSM_TP_HANDLE TPHandle, 
                                          CSSM_CL_HANDLE CLHandle, 
                                          CSSM_DL_HANDLE DLHandle, 
                                          CSSM_DB_HANDLE DBHandle, 
                                          CSSM_CC_HANDLE CCHandle, 
                                          uint32 PassThroughId,
                                          const CSSM_DATA_PTR InputParams);
    CSSM_RETURN (CSSMAPI *Initialize) (uint32 VerMajor,
                                       uint32 VerMinor);
    CSSM_RETURN (CSSMAPI *Uninitialize) (void);
} CSSM_SPI_TP_FUNCS, *CSSM_SPI_TP_FUNCS_PTR;

/* Trust Policy SPI */
CSSM_RETURN CSSMAPI CSSM_TP_RegisterServices 
                                     (const CSSM_GUID_PTR GUID,
                                      const CSSM_SPI_TP_FUNCS_PTR FunctionTable,
                                      CSSM_SPI_MEMORY_FUNCS_PTR UpcallTable,
                                      void *Reserved);
CSSM_RETURN CSSMAPI CSSM_TP_DeregisterServices (const CSSM_GUID_PTR GUID);

#ifdef __cplusplus
}
#endif

#endif
