/*-----------------------------------------------------------------------
 *      File:   CSSMTYPE.H
 *
 * Copyright (c) 1995, 1996, 1997 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
/* 
 * WARNING: EXPORT RESTRICTED. 
 * This software listing contains cryptographic methods and technology. 
 * It is export restricted by the Office of Defense Trade Controls, United 
 * States Department of State and cannot be downloaded or otherwise 
 * exported or re-exported (i) into (or to a national or resident of) Cuba, 
 * Iraq, Libya, Yugoslavia, North Korea, Iran, Syria or any other country 
 * to which the US has embargoed goods; or (ii) to anyone on the US 
 * Treasury Department's list of Specially Designated Nationals or the US 
 * Commerce Department's Table of Denial Orders. By downloading or using 
 * this product, you are agreeing to the foregoing and you are representing 
 * and warranting that you are not located in, under the control of, or a 
 * national or resident of any such country or on any such list. 
 */ 

#ifndef _CSSMTYPE_H
#define _CSSMTYPE_H

/* Operating System Dependent Primative Declarations */
#if defined (WIN32)

#define CSSMAPI WINAPI

#elif defined (WIN31)

#ifndef PASCAL
#define PASCAL __stdcall
#endif
#define CSSMAPI PASCAL

#elif defined (UNIX) 

#define CSSMAPI

#endif


#if defined (WIN32)

#include <windows.h>

/* Basic Types */
typedef unsigned char uint8;
typedef unsigned short uint16;
typedef short sint16;
typedef unsigned int uint32;
typedef int sint32;

/* Thread types */
#define CSSM_THREAD __declspec(thread);

#endif /* defined(WIN32) */


#define MAXBUFFSIZE        1024
#define MAX_KEYBLOB_LEN    1024


typedef uint32 CSSM_HANDLE, *CSSM_HANDLE_PTR;
typedef uint32 CSSM_TP_HANDLE;    /* Trust Policy Handle */
typedef uint32 CSSM_CL_HANDLE;    /* Certificate Library Handle */
typedef uint32 CSSM_DL_HANDLE;    /* Database Library Handle */
typedef uint32 CSSM_DB_HANDLE;    /* Database Handle */
typedef uint32 CSSM_CC_HANDLE;    /* Cryptographic Context Handle */
typedef uint32 CSSM_CSP_HANDLE;   /* Cryptographic Service Provider Handle */
typedef uint32 CSSM_MODULE_HANDLE, *CSSM_MODULE_HANDLE_PTR /* Service provider
                                                            Handle*/


typedef uint32 CSSM_TP_ACTION;

typedef uint32 CSSM_EVENT_TYPE, *CSSM_EVENT_TYPE_PTR;

#define CSSM_EVENT_ATTACH        (0)
#define CSSM_EVENT_DETACH        (1)
#define CSSM_EVENT_INFOATTACH    (2)
#define CSSM_EVENT_INFODETACH    (3)

typedef uint32 CSSM_KEY_TYPE, *CSSM_KEY_TYPE_PTR;

typedef enum cssm_bool {
    CSSM_TRUE = 1,
    CSSM_FALSE = 0
} CSSM_BOOL;

typedef enum cssm_return {
    CSSM_OK = 0,
    CSSM_FAIL = -1
} CSSM_RETURN;

typedef struct cssm_guid {
    uint32 Data1;
    uint16 Data2;
    uint16 Data3;
    uint8 Data4[8];
} CSSM_GUID, *CSSM_GUID_PTR;

typedef struct cssm_error {
    uint32 error;
    CSSM_GUID guid;
} CSSM_ERROR, *CSSM_ERROR_PTR;

typedef enum cssm_revoke_reason { 
    CSSM_REVOKE_CUSTOM,
    CSSM_REVOKE_UNSPECIFIC,
    CSSM_REVOKE_KEYCOMPROMISE,
    CSSM_REVOKE_CACOMPROMISE,
    CSSM_REVOKE_AFFILIATIONCHANGED,
    CSSM_REVOKE_SUPERCEDED,
    CSSM_REVOKE_CESSATIONOFOPERATION,
    CSSM_REVOKE_CERTIFICATEHOLD,
    CSSM_REVOKE_CERTIFICATEHOLDRELEASE,
    CSSM_REVOKE_REMOVEFROMCRL
} CSSM_REVOKE_REASON;

typedef struct cssm_data {
    uint32 Length;            /* in bytes */
    uint8 *Data;
} CSSM_DATA, *CSSM_DATA_PTR;

typedef CSSM_DATA CSSM_OID, *CSSM_OID_PTR;

typedef enum cssm_cert_type {
    CSSM_CERT_UNKNOWN=   0x00,
    CSSM_CERT_X_509v1 =  0x01,
    CSSM_CERT_X_509v2 =  0x02,
    CSSM_CERT_X_509v3 =  0x03,
    CSSM_CERT_Fortezza = 0x07,
    CSSM_CERT_PGP =      0x04,
    CSSM_CERT_SPKI =     0x05,
    CSSM_CERT_SDSIv1 =   0x06,
    CSSM_CERT_Intel =    0x08,
    CSSM_CERT_ATTRIBUTE_BER = 0x09, /* ber encoded X.509 attribute cert */
    CSSM_CERT_LAST =     0xFF
} CSSM_CERT_TYPE, *CSSM_CERT_TYPE_PTR;

/* Applications wishing to define their own custom certificate type should 
 * create a random uint32 whose value is greater than the CSSM_CL_CUSTOM_CERT_TYPE */
#define CSSM_CL_CUSTOM_CERT_TYPE  0x0100 

typedef struct {
    CSSM_DL_HANDLE DLHandle;
    CSSM_DB_HANDLE DBHandle;
} CSSM_DB_LONGHANDLE, *CSSM_DB_LONGHANDLE_PTR;

typedef struct {
    uint32 NumHandles;
    CSSM_DB_LONGHANDLE_PTR DBLongHandle;
} CSSM_DB_LIST, *CSSM_DB_LIST_PTR;

typedef struct
{
    uint32 NumCerts;
    CSSM_DATA_PTR CertList;
    void* reserved;
} CSSM_CERTGROUP, *CSSM_CERTGROUP_PTR;

/* enumeration of data store types */
typedef enum {
    CSSM_DB_TAG_NONE = 0L,
    CSSM_DB_TAG_ROOTS, /* contains self-signed root certs */
    CSSM_DB_TAG_TRUSTED, /* re-issued locally (DL must protect?) */
    CSSM_DB_TAG_SYSTEM, /* contains CSSM system certs */
    CSSM_DB_TAG_OWNER, /* certs owned by users (ie private key in CSP) */
    CSSM_DB_TAG_REVOKED /* contains revocation information - used with CRL APIs */
    /* others? */
} CSSM_DB_TYPE;

/* data record types stored and managed by data storage library modules */
typedef enum  cssm_data_record_type {
    CSSM_DATA_RECORD_ANY,        /* lib can store and manage any objects */
    CSSM_DATA_RECORD_CERTS,        /* lib can store & manage certs */
    CSSM_DATA_RECORD_CRLS,        /* lib can store & manage CRLS */
    CSSM_DATA_RECORD_KEYS,        /* lib can store & manage keys */
    CSSM_DATA_RECORD_POLICIES,        /* lib can store & manage policies */
    CSSM_DATA_RECORD_GENERIC        /* lib can store & manage generic data */
} CSSM_DATA_RECORD_TYPE, *CSSM_DATA_RECORD_TYPE_PTR;

/* Attribute of a data store record can be treated by the dl
as following types */. 
typedef enum  cssm_db_attribute_usage {
    CSSM_DB_INDEX,
    CSSM_DB_UNIQUE_INDEX,
    CSSM_DB_NON_INDEX
}   CSSM_DB_ATTRIBUTE_USAGE, *CSSM_DB_ATTRIBUTE_USAGE_PTR;

/* Distinct representations for an attribute identifier */
typedef enum cssm_attribute_id_format { 
    CSSM_OID_NAME_FORMAT, 
    CSSM_STRING_NAME_FORMAT 
} CSSM_ATTRIBUTE_ID_FORMAT, *CSSM_ATTRIBUTE_ID_FORMAT_PTR;

typedef struct cssm_attribute_name {
    CSSM_ATTRIBUTE_ID_FORMAT AttributeIdFormat;
    CSSM_DATA AttributeId;
} CSSM_ATTRIBUTE_NAME, *CSSM_ATTRIBUTE_NAME_PTR;

typedef struct {
    CSSM_DB_ATTRIBUTE_NAME  AttributeName;     
    CSSM_DB_ATTRIBUTE_USAGE  AttributeUsage;
}  CSSM_DB_ATTRIBUTE_INFO, CSSM_ATTRIBUTE_INFO_PTR;

/* Meta-information about an individual record type in a single data store */
typedef struct {
    CSSM_DATA_RECORD_TYPE DataRecordType;
    uint32 NumberOfAttributes;
    CSSM_DB_ATTRIBUTE_INFO_PTR AttributeInfo; 
} CSSM_DB_RECORD_INFO,  *CSSM_DB_RECORD_INFO_PTR;

/* Meta-information about an individual data store. */
typedef struct {
    CSSM_DB_TYPE DbType;
    uint32 NumberOfRecordTypes;
    CSSM_DB_RECORD_INFO_PTR RecordInfo;
    void *reserved; 
} CSSM_DBINFO,  *CSSM_DBINFO_PTR;

/* Conjunctive operations*/
typedef enum cssm_db_conjunctive{ 
     CSSM_NONE
     CSSM_AND, 
     CSSM_OR
} CSSM_DB_CONJUNCTIVE

/* logical operators */
typedef enum cssm_db_operator { 
    CSSM_EQUAL, 
    CSSM_NOT_EQUAL, 
    CSSM_LESS_THAN, 
    CSSM_GREATER_THAN 
} CSSM_DB_OPERATOR;

/* Tag to specify whether the data object retrieval is based on 
 query string or selection predicates */
typedef enum cssm_tag_query { 
    CSSM_QUERY_NONE,
    CSSM_QUERY_STRING, 
    CSSM_QUERY_PREDICATES 
} CSSM_QUERY_TAG;

/* Selection predicate to be used for database queries */
typedef struct cssm_selection_predicate {
    CSSM_DB_OPERATOR dbOperator;
    CSSM_ATTRIBUTE_NAME AttributeName;
    CSSM_DATA AttributeValue;
} CSSM_SELECTION_PREDICATE, *CSSM_SELECTION_PREDICATE_PTR;

/* Query predicate to be used for database queries */
 typedef struct cssm_query_predicate {
    CSSM_DB_CONJUNCTIVE Conjunctive;
    uint32 NumSelectionPredicates;
    CSSM_SELECTION_PREDICATE_PTR SelectionPredicate;
} CSSM_QUERY_PREDICATE, *CSSM_QUERY_PREDICATE_PTR;

/* Union  contains the array of selection predicates and a CSSM data ptr.
Depending on the CSSM_QUERY_TAG, one of this has to be selected. */

typedef union cssm_query {
    CSSM_DATA QueryString;
    CSSM_QUERY_PREDICATE_PTR QueryPredicate;
} CSSM_QUERY, *CSSM_QUERY_PTR;

/* Defines the index values to be inserted or deleted */
typedef struct cssm_index_record {
    CSSM_ATTRIBUTE_NAME IndexName;
    CSSM_DATA IndexValue;
} CSSM_INDEX_RECORD, *CSSM_INDEX_RECORD_PTR;

/* module implementation types for data storage library modules */

typedef uint32 CSSM_DL_MODULE_TYPE

#define CSSM_STORE_LOCAL_MEMORY 0x00000001     /* implementation uses a local
                                            memory cache */
#define CSSM_STORE_LOCAL_FILE   0x00000002    /* implement. uses a file system
                                            service */
#define CSSM_STORE_LOCAL_DBMS   0x00000004    /* implementation uses a local
                                            DBMS */
#define CSSM_STORE_REMOTE_DBMS  0x00000008     /* implementation uses a remote 
                                            DBMS */
#define CSSM_STORE_REMOTE_DIR   0x00000010    /* implementation uses a remote
                                            directory service */
#define CSSM_STORE_TOKEN        0x00000020    /* implementation uses a  
                                            hardware token */
#define CSSM_STORE_REMOVEABLE   0x00000040    /* implementation uses a removable 
                                            store device */
#define CSSM_STORE_UNSPECIFIED  0x00000080    /* implementation is unspecified 
                                        */

typedef enum cssm_algorithms {
    CSSM_ALGID_NONE      = 0,
    CSSM_ALGID_CUSTOM    = CSSM_ALGID_NONE+1, /* Custom algorithm */
    CSSM_ALGID_DH        = CSSM_ALGID_NONE+2, /* Diffie Hellman key exchange algorithm */
    CSSM_ALGID_PH        = CSSM_ALGID_NONE+3, /* Pohlig Hellman key exchange algorithm */
    CSSM_ALGID_KEA       = CSSM_ALGID_NONE+4, /* Key Exchange Algorithm */
    CSSM_ALGID_MD2       = CSSM_ALGID_NONE+5, /* MD2  hash algorithm  (invented by Ron Rivest) */
    CSSM_ALGID_MD4       = CSSM_ALGID_NONE+6, /* MD4  hash algorithm  (invented by Ron Rivest) */
    CSSM_ALGID_MD5       = CSSM_ALGID_NONE+7, /* MD5  hash algorithm  (invented by Ron Rivest) */
    CSSM_ALGID_SHA1      = CSSM_ALGID_NONE+8, /* Secure Hash Algorithm  (developed by NIST/NSA) */
    CSSM_ALGID_NHASH     = CSSM_ALGID_NONE+9, /* N-Hash algorithm(developed by Nippon Telephone and Telegraph) */
    CSSM_ALGID_HAVAL     = CSSM_ALGID_NONE+10,/* HAVAL  hash algorithm  (MD5 variant) */
    CSSM_ALGID_RIPEMD    = CSSM_ALGID_NONE+11,/* RIPE-MD  (160) hash algorithm  (MD4 variant - developed for the European Community's RIPE project) */
    CSSM_ALGID_IBCHASH   = CSSM_ALGID_NONE+12,/* IBC-Hash (keyed hash algorithm or MAC) */
    CSSM_ALGID_RIPEMAC   = CSSM_ALGID_NONE+13,/* RIPE-MAC (invented by Bart Preneel) */
    CSSM_ALGID_DES       = CSSM_ALGID_NONE+14,/* Data Encryption Standard block cipher */
    CSSM_ALGID_DESX      = CSSM_ALGID_NONE+15,/* DESX block cipher  (DES variant from RSA) */
    CSSM_ALGID_RDES      = CSSM_ALGID_NONE+16,/* RDES block cipher  (DES variant) */
    CSSM_ALGID_3DES_3KEY = CSSM_ALGID_NONE+17,/* Triple-DES block cipher  (with 3 keys) */
    CSSM_ALGID_3DES_2KEY = CSSM_ALGID_NONE+18,/* Triple-DES block cipher  (with 2 keys) */
    CSSM_ALGID_3DES_1KEY = CSSM_ALGID_NONE+19,/* Triple-DES block cipher  (with 1 key) */
    CSSM_ALGID_IDEA      = CSSM_ALGID_NONE+20,/* IDEA block cipher  (invented by Lai and Massey) */
    CSSM_ALGID_RC2       = CSSM_ALGID_NONE+21,/* RC2 block cipher  (invented by Ron Rivest) */
    CSSM_ALGID_RC5       = CSSM_ALGID_NONE+22,/* RC5 block cipher  (invented by Ron Rivest) */
    CSSM_ALGID_RC4       = CSSM_ALGID_NONE+23,/* RC4 stream cipher  (invented by Ron Rivest) */
    CSSM_ALGID_SEAL      = CSSM_ALGID_NONE+24,/* SEAL stream cipher  (invented by Rogaway and Coppersmith) */
    CSSM_ALGID_CAST      = CSSM_ALGID_NONE+25,/* CAST block cipher  (invented by Adams and Tavares) */
    CSSM_ALGID_BLOWFISH  = CSSM_ALGID_NONE+26,/* BLOWFISH block cipher  (invented by Schneier) */
    CSSM_ALGID_SKIPJACK  = CSSM_ALGID_NONE+27,/* Skipjack block cipher  (developed by NSA) */
    CSSM_ALGID_LUCIFER   = CSSM_ALGID_NONE+28,/* Lucifer block cipher  (developed by IBM) */
    CSSM_ALGID_MADRYGA   = CSSM_ALGID_NONE+29,/* Madryga block cipher  (invented by Madryga) */
    CSSM_ALGID_FEAL      = CSSM_ALGID_NONE+30,/* FEAL block cipher  (invented by Shimizu and Miyaguchi) */
    CSSM_ALGID_REDOC     = CSSM_ALGID_NONE+31,/* REDOC 2 block cipher  (invented by Michael Wood) */
    CSSM_ALGID_REDOC3    = CSSM_ALGID_NONE+32,/* REDOC 3 block cipher  (invented by Michael Wood) */
    CSSM_ALGID_LOKI      = CSSM_ALGID_NONE+33,/* LOKI block cipher */
    CSSM_ALGID_KHUFU     = CSSM_ALGID_NONE+34,/* KHUFU block cipher  (invented by Ralph Merkle) */
    CSSM_ALGID_KHAFRE    = CSSM_ALGID_NONE+35,/* KHAFRE block cipher  (invented by Ralph Merkle) */
    CSSM_ALGID_MMB       = CSSM_ALGID_NONE+36,/* MMB block cipher  (IDEA variant) */
    CSSM_ALGID_GOST      = CSSM_ALGID_NONE+37,/* GOST block cipher  (developed by the former Soviet Union) */
    CSSM_ALGID_SAFER     = CSSM_ALGID_NONE+38,/* SAFER K-64 block cipher  (invented by Massey) */
    CSSM_ALGID_CRAB      = CSSM_ALGID_NONE+39,/* CRAB block cipher  (invented by Kaliski and Robshaw) */
    CSSM_ALGID_RSA       = CSSM_ALGID_NONE+40,/* RSA public key cipher */
    CSSM_ALGID_DSA       = CSSM_ALGID_NONE+41,/* Digital Signature Algorithm */
    CSSM_ALGID_MD5WithRSA= CSSM_ALGID_NONE+42,/* MD5/RSA signature algorithm */
    CSSM_ALGID_MD2WithRSA= CSSM_ALGID_NONE+43,/* MD2/RSA signature algorithm */
    CSSM_ALGID_ElGamal   = CSSM_ALGID_NONE+44,/* ElGamal signature algorithm */
    CSSM_ALGID_MD2Random = CSSM_ALGID_NONE+45,/* MD2-based random numbers */
    CSSM_ALGID_MD5Random = CSSM_ALGID_NONE+46,/* MD5-based random numbers */
    CSSM_ALGID_SHARandom = CSSM_ALGID_NONE+47,/* SHA-based random numbers */
    CSSM_ALGID_DESRandom = CSSM_ALGID_NONE+48, /* DES-based random numbers */
    CSSM_ALGID_SHA1WithRSA= CSSM_ALGID_NONE+49, /* SHA1/RSA signature algorithm */
    CSSM_ALGID_RSA_PKCS  = CSSM_ALGID_RSA,     /* RSA as specified in PKCS #1 */
    CSSM_ALGID_RSA_ISO9796 = CSSM_ALGID_NONE+50, /* RSA as specified in ISO 9796 */
    CSSM_ALGID_RSA_RAW   = CSSM_ALGID_NONE+51, /* Raw RSA as assumed in X.509 */
    CSSM_ALGID_CDMF      = CSSM_ALGID_NONE+52, /* CDMF block cipher */
    CSSM_ALGID_CAST3     = CSSM_ALGID_NONE+53, /* Entrust's CAST3 block cipher */
    CSSM_ALGID_CAST5     = CSSM_ALGID_NONE+54, /* Entrust's CAST5 block cipher */
    CSSM_ALGID_GenericSecret = CSSM_ALGID_NONE+55, /* Generic secret operations */
    CSSM_ALGID_ConcatBaseAndKey = CSSM_ALGID_NONE+56, /* Concatenate two keys, base key first */
    CSSM_ALGID_ConcatKeyAndBase = CSSM_ALGID_NONE+57, /* Concatenate two keys, base key last */
    CSSM_ALGID_ConcatBaseAndData = CSSM_ALGID_NONE+58, /* Concatenate base key and random data, key first */
    CSSM_ALGID_ConcatDataAndBase = CSSM_ALGID_NONE+59, /* Concatenate base key and data, data first */
    CSSM_ALGID_XORBaseAndData = CSSM_ALGID_NONE+60, /* XOR a byte string with the base key */
    CSSM_ALGID_ExtractFromKey = CSSM_ALGID_NONE+61, /* Extract a key from base key, starting at arbitrary bit position */
    CSSM_ALGID_SSL3PreMasterGen = CSSM_ALGID_NONE+62, /* Generate a 48 byte SSL 3 pre-master key */
    CSSM_ALGID_SSL3MasterDerive = CSSM_ALGID_NONE+63, /* Derive an SSL 3 key from a pre-master key */
    CSSM_ALGID_SSL3KeyAndMacDerive = CSSM_ALGID_NONE+64, /* Derive the keys and MACing keys for the SSL cipher suite */
    CSSM_ALGID_SSL3MD5_MAC = CSSM_ALGID_NONE+65, /* Performs SSL 3 MD5 MACing */
    CSSM_ALGID_SSL3SHA1_MAC = CSSM_ALGID_NONE+66, /* Performs SSL 3 SHA-1 MACing */
    CSSM_ALGID_MD5Derive = CSSM_ALGID_NONE+67, /* Generate key by MD5 hashing a base key */
    CSSM_ALGID_MD2Derive = CSSM_ALGID_NONE+68, /* Generate key by MD2 hashing a base key */
    CSSM_ALGID_SHA1Derive = CSSM_ALGID_NONE+69, /* Generate key by SHA-1 hashing a base key */
    CSSM_ALGID_WrapLynks = CSSM_ALGID_NONE+70, /* Spyrus LYNKS DES based wrapping scheme w/checksum */
    CSSM_ALGID_WrapSET_OAEP = CSSM_ALGID_NONE+71, /* SET key wrapping */
    CSSM_ALGID_BATON    = CSSM_ALGID_NONE+72, /* Fortezza BATON cipher */
    CSSM_ALGID_ECDSA    = CSSM_ALGID_NONE+73, /* Elliptic Curve DSA */
    CSSM_ALGID_MAYFLY   = CSSM_ALGID_NONE+74, /* Fortezza MAYFLY cipher */
    CSSM_ALGID_JUNIPER  = CSSM_ALGID_NONE+75, /* Fortezza JUNIPER cipher */
    CSSM_ALGID_FASTHASH = CSSM_ALGID_NONE+76, /* Fortezza FASTHASH */
} CSSM_ALGORITHMS;

typedef enum cssm_encrypt_mode {
    CSSM_ALGMODE_NONE       = 0,
    CSSM_ALGMODE_CUSTOM     = CSSM_ALGMODE_NONE+1, /* Custom mode */
    CSSM_ALGMODE_ECB        = CSSM_ALGMODE_NONE+2, /* Electronic Code Book */
    CSSM_ALGMODE_ECBPad     = CSSM_ALGMODE_NONE+3, /* ECB with padding */
    CSSM_ALGMODE_CBC        = CSSM_ALGMODE_NONE+4, /* Cipher Block Chaining */
    CSSM_ALGMODE_CBC_IV8    = CSSM_ALGMODE_NONE+5, /* CBC with Initialization Vector of 8 bytes */
    CSSM_ALGMODE_CBCPadIV8  = CSSM_ALGMODE_NONE+6, /* CBC with padding and Initialization Vector of 8 bytes */
    CSSM_ALGMODE_CFB        = CSSM_ALGMODE_NONE+7, /* Cipher FeedBack */
    CSSM_ALGMODE_CFB_IV8    = CSSM_ALGMODE_NONE+8, /* CFB with Initialization Vector of 8 bytes */
    CSSM_ALGMODE_OFB        = CSSM_ALGMODE_NONE+9, /* Output FeedBack */
    CSSM_ALGMODE_OFB_IV8    = CSSM_ALGMODE_NONE+10,/* OFB with Initialization Vector of 8 bytes */
    CSSM_ALGMODE_COUNTER    = CSSM_ALGMODE_NONE+11,/* Counter */
    CSSM_ALGMODE_BC         = CSSM_ALGMODE_NONE+12,/* Block Chaining */
    CSSM_ALGMODE_PCBC       = CSSM_ALGMODE_NONE+13,/* Propagating CBC */
    CSSM_ALGMODE_CBCC       = CSSM_ALGMODE_NONE+14,/* CBC with Checksum */
    CSSM_ALGMODE_OFBNLF     = CSSM_ALGMODE_NONE+15,/* OFB with NonLinear Function */
    CSSM_ALGMODE_PBC        = CSSM_ALGMODE_NONE+16,/* Plaintext Block Chaining */
    CSSM_ALGMODE_PFB        = CSSM_ALGMODE_NONE+17,/* Plaintext FeedBack */
    CSSM_ALGMODE_CBCPD      = CSSM_ALGMODE_NONE+18,/* CBC of Plaintext Difference */
    CSSM_ALGMODE_PUBLIC_KEY = CSSM_ALGMODE_NONE+19,/* Encrypt with the public key */
    CSSM_ALGMODE_PRIVATE_KEY= CSSM_ALGMODE_NONE+20,/* Encrypt with the private key */
    CSSM_ALGMODE_SHUFFLE    = CSSM_ALGMODE_NONE+21 /* Fortezza shuffle mode */
} CSSM_ENCRYPT_MODE;

typedef enum cssm_context_type {
    CSSM_ALGCLASS_NONE      = 0,
    CSSM_ALGCLASS_CUSTOM    = CSSM_ALGCLASS_NONE+1,
    CSSM_ALGCLASS_KEYXCH    = CSSM_ALGCLASS_NONE+2,
    CSSM_ALGCLASS_SIGNATURE = CSSM_ALGCLASS_NONE+3,
    CSSM_ALGCLASS_SYMMETRIC = CSSM_ALGCLASS_NONE+4,
    CSSM_ALGCLASS_DIGEST    = CSSM_ALGCLASS_NONE+5,
    CSSM_ALGCLASS_RANDOMGEN = CSSM_ALGCLASS_NONE+6,
    CSSM_ALGCLASS_UNIQUEGEN = CSSM_ALGCLASS_NONE+7,
    CSSM_ALGCLASS_MAC       = CSSM_ALGCLASS_NONE+8,
    CSSM_ALGCLASS_ASYMMETRIC= CSSM_ALGCLASS_NONE+9,
    CSSM_ALGCLASS_KEYGEN    = CSSM_ALGCLASS_NONE+10,
    CSSM_ALGCLASS_DERIVEKEY = CSSM_ALGCLASS_NONE+11
}CSSM_CONTEXT_TYPE;


typedef enum cssm_attribute_type {
    CSSM_ATTRIBUTE_NONE         = 0,
    CSSM_ATTRIBUTE_CUSTOM       = CSSM_ATTRIBUTE_NONE+1,
    CSSM_ATTRIBUTE_DESCRIPTION  = CSSM_ATTRIBUTE_NONE+2,
    CSSM_ATTRIBUTE_KEY          = CSSM_ATTRIBUTE_NONE+3,
    CSSM_ATTRIBUTE_INIT_VECTOR  = CSSM_ATTRIBUTE_NONE+4,
    CSSM_ATTRIBUTE_SALT         = CSSM_ATTRIBUTE_NONE+5,
    CSSM_ATTRIBUTE_PADDING      = CSSM_ATTRIBUTE_NONE+6,
    CSSM_ATTRIBUTE_RANDOM       = CSSM_ATTRIBUTE_NONE+7,
    CSSM_ATTRIBUTE_SEED         = CSSM_ATTRIBUTE_NONE+8,
    CSSM_ATTRIBUTE_PASSPHRASE   = CSSM_ATTRIBUTE_NONE+9,
    CSSM_ATTRIBUTE_KEY_LENGTH   = CSSM_ATTRIBUTE_NONE+10,
    CSSM_ATTRIBUTE_MODULUS_LEN  = CSSM_ATTRIBUTE_NONE+11,
    CSSM_ATTRIBUTE_INPUT_SIZE   = CSSM_ATTRIBUTE_NONE+12,
    CSSM_ATTRIBUTE_OUTPUT_SIZE  = CSSM_ATTRIBUTE_NONE+13,
    CSSM_ATTRIBUTE_ROUNDS       = CSSM_ATTRIBUTE_NONE+14
} CSSM_ATTRIBUTE_TYPE;

/*typedef enum cssm_keyblob_type {
    CSSM_SESSION_KEY_BLOB       = 0,
    CSSM_RSA_PUBLIC_KEY_BLOB    = CSSM_SESSION_KEY_BLOB+1,
    CSSM_RSA_PRIVATE_KEY_BLOB   = CSSM_SESSION_KEY_BLOB+2,
    CSSM_DSA_PUBLIC_KEY_BLOB    = CSSM_SESSION_KEY_BLOB+3,
    CSSM_DSA_PRIVATE_KEY_BLOB   = CSSM_SESSION_KEY_BLOB+4
} CSSM_KEYBLOB_TYPE;
*/

/*typedef enum cssm_keywrap_method {
    CSSM_KEYWRAP_NONE           = 0,
    CSSM_KEYWRAP_MD5WithDES     = CSSM_KEYWRAP_NONE+1,
    CSSM_KEYWRAP_MD5WithIDEA    = CSSM_KEYWRAP_NONE+2,
    CSSM_KEYWRAP_SHA1WithDES    = CSSM_KEYWRAP_NONE+3,
    CSSM_KEYWRAP_SHA1WithIDEA   = CSSM_KEYWRAP_NONE+4
} CSSM_KEYWRAP_METHOD;
*/

typedef enum cssm_padding { 
    CSSM_PADDING_NONE           = 0,
    CSSM_PADDING_CUSTOM         = CSSM_PADDING_NONE+1,
    CSSM_PADDING_ZERO           = CSSM_PADDING_NONE+2,
    CSSM_PADDING_ONE            = CSSM_PADDING_NONE+3,
    CSSM_PADDING_ALTERNATE      = CSSM_PADDING_NONE+4,
    CSSM_PADDING_FF             = CSSM_PADDING_NONE+5,
    CSSM_PADDING_PKCS7          = CSSM_PADDING_NONE+6,
    CSSM_PADDING_CipherStealing = CSSM_PADDING_NONE+7,
    CSSM_PADDING_RANDOM         = CSSM_PADDING_NONE+8
} CSSM_PADDING;

typedef CSSM_DATA_PTR (CALLBACK *CSSM_CALLBACK) (void *allocRef, uint32 ID);

typedef struct CSSM_KeyHeader {
    uint32 HeaderFormatVersion; /* Version of the current header format */
    CSSM_GUID CspId;            /* GUID of CSP that generated the key */
    uint32 BlobDescription;     /* Mask describing the key data format */
    uint32 DataFormatVersion;   /* Version of the key data format */
    uint32 AlgorithmId;         /* ALGID of the creation algorithm */
    uint32 KeyUsage;            /* Mask of authorized key usages */
    uint32 SizeInBits;          /* key size in bits */
    uint32 WrapMethod;          /* ALGID of the mechanism used to wrap the key */
    uint32 Reserved;            /* Reserved for future use */
} CSSM_KEYHEADER, *CSSM_KEYHEADER_PTR ;

#define CSSM_KEYHEADER_VERSION    (1)
#define CSSM_DATAFORMAT_VERSION   (1)

/* When creating BlobDescription Mask, use one from each of the following groups */
/* Wrap state */
#define CSSM_BLOBDESC_WRAP_MASK      (0x80000000) /* Use to mask wrap flag */
#define CSSM_BLOBDESC_UNWRAPPED      (0x00000000) /* Key is encrypted, might not be parsable */
#define CSSM_BLOBDESC_WRAPPED        (0x80000000) /* Key is cleartext, can be parsed */
/* Transient State */
#define CSSM_BLOBDESC_TRANS_MASK     (0x40000000) /* Use to mask the transient bit */
#define CSSM_BLOBDESC_PERMANENT      (0x00000000) /* Key data is constant across attaches */
#define CSSM_BLOBDESC_TRANSIENT      (0x40000000) /* Key data is not constant across attaches */
/* Data Type */
#define CSSM_BLOBDESC_TYPE_MASK      (0x30000000) /* Use to mask type value */
#define CSSM_BLOBDESC_DATA           (0x00000000) /* Actual key data is present */
#define CSSM_BLOBDESC_LABEL          (0x10000000) /* Lebel reference to key present */
#define CSSM_BLOBDESC_HANDLE         (0x20000000) /* Handle reference to key present */
/* Contents */
#define CSSM_BLOBDESC_CONTENTS_MASK (0x0F000000) /* Use to mask contents value */
#define CSSM_BLOBDESC_PUBLIC_KEY    (0x00000000)
#define CSSM_BLOBDESC_PRIVATE_KEY   (0x01000000)
#define CSSM_BLOBDESC_SESSION_KEY   (0x02000000)
#define CSSM_BLOBDESC_SECRET_PART   (0x03000000) /* Part of shared secret */
/* Data Format */
#define CSSM_BLOBDESC_FORMAT_MASK   (0x00FFFF00) /* Use to mask format value */
#define CSSM_BLOBDESC_RAW           (0x00000000) /* Single part key data, no encoding */
#define CSSM_BLOBDESC_BER           (0x00000100)
#define CSSM_BLOBDESC_PKCS1         (0x00000200) /* RSA Inc. PKCS #1 - RSA */
#define CSSM_BLOBDESC_PKCS3         (0x00000300) /* RSA Inc. PKCS #3 - Diffie Hellman */
#define CSSM_BLOBDESC_MSCAPI        (0x00000400) /* Microsoft CAPI */
#define CSSM_BLOBDESC_PGP           (0x00000500)
#define CSSM_BLOBDESC_FIPS186       (0x00000600) /* FIPS Pub 186 - DSS */

/* Key usage masks */
#define CSSM_KEYUSE_ENCRYPT         0x0001
#define CSSM_KEYUSE_DECRYPT         0x0002
#define CSSM_KEYUSE_SIGN            0x0004
#define CSSM_KEYUSE_VERIFY          0x0008
#define CSSM_KEYUSE_WRAP            0x0010
#define CSSM_KEYUSE_UNWRAP          0x0020
#define CSSM_KEYUSE_DERIVE          0x0040

typedef struct cssm_keyblob {
    CSSM_KEYHEADER KeyHeader;
    uint8 KeyData [MAX_KEYBLOB_LEN];
} CSSM_KEYBLOB, *CSSM_KEYBLOB_PTR;

/*typedef struct cssm_key {
    uint32 KeyBlobLength;
    CSSM_KEYBLOB_PTR KeyBlob;
} CSSM_KEY, *CSSM_KEY_PTR;
*/
typedef CSSM_DATA CSSM_KEY, *CSSM_KEY_PTR;
typedef CSSM_KEY CSSM_WRAP_KEY, *CSSM_WRAP_KEY_PTR;

typedef struct cssm_crypto_data {
    CSSM_DATA_PTR Param;
    CSSM_CALLBACK Callback;
    uint32 ID;
} CSSM_CRYPTO_DATA, *CSSM_CRYPTO_DATA_PTR;

typedef struct cssm_list_item {
    CSSM_GUID GUID;
    char *Name;
} CSSM_LIST_ITEM, *CSSM_LIST_ITEM_PTR;

typedef struct cssm_list {
    uint32 NumberItems;
    CSSM_LIST_ITEM_PTR Items;
} CSSM_LIST, *CSSM_LIST_PTR;

typedef struct cssm_name_list {
    uint32 NumStrings;
    char** String;
} CSSM_NAME_LIST, *CSSM_NAME_LIST_PTR;

typedef struct cssm_info {
    uint32 VerMajor;
    uint32 VerMinor;
}CSSM_INFO, *CSSM_INFO_PTR;

typedef struct cssm_context_attribute{
    uint32 AttributeType;    /* one of the defined CSSM_ATTRIBUTE_TYPEs */
    uint32 AttributeLength;  /* length of attribute */
    union {
        uint8 *Description;
        uint32 *Length;
        void *Pointer;
        CSSM_CRYPTO_DATA_PTR SeedPassPhrase;
        CSSM_KEY_PTR Key;
        CSSM_DATA_PTR Data;
    } Attribute;            /* data that describes attribute */
} CSSM_CONTEXT_ATTRIBUTE, *CSSM_CONTEXT_ATTRIBUTE_PTR;

typedef struct cssm_context {
    uint32 ContextType;        /* one of the defined CSSM_CONTEXT_TYPEs */
    uint32 AlgorithmType;      /* one of the defined CSSM_ALGORITHMSs */
    uint32 Mode;               /* one of the defined CSSM_ENCRYPT_MODEs - for encryption only */
    uint32 Reserve;            /* reserved for future use */
    uint32 NumberOfAttributes; /* number of attributes associated with context */
    CSSM_CONTEXT_ATTRIBUTE_PTR ContextAttributes;    /* pointer to attributes */
} CSSM_CONTEXT, *CSSM_CONTEXT_PTR;

typedef CSSM_CONTEXT CSSM_CONTEXTINFO;

typedef enum cssm_csp_type {
    CSSM_CSPTYPE_HW     = 0, 
    CSSM_CSPTYPE_SW     = CSSM_CSPTYPE_HW+1,
    CSSM_CSPTYPE_HYBRID = CSSM_CSPTYPE_HW+2
} CSSM_CSP_TYPE;

typedef struct cssm_csppininfo {
    uint32 MaxLength;
    uint32 MinLength;
} CSSM_CSPPININFO, *CSSM_CSPPININFO_PTR;

typedef struct cssm_cspmeminfo {
    uint32 PublicMem;
    uint32 FreePublicMem;
    uint32 PrivateMem;
    uint32 FreePrivateMem;
} CSSM_CSPMEMINFO, *CSSM_CSPMEMINFO_PTR;

typedef struct cssm_cspsessioninfo {
    uint32 MaxSessions;
    uint32 OpenedSessions;
    uint32 MaxRWSessions;
    uint32 OpenedRWSessions;
} CSSM_CSPSESSIONINFO, CSSM_CSPSESSIONINFO_PTR;

typedef struct cssm_cspinfo {
    uint32 VerMajor;                /* Major version of the token */
    uint32 VerMinor;                /* Minor version of the token */
    CSSM_BOOL ExportFlag;           /* Exportable outside the U.S. */
    CSSM_BOOL MultiTasking;         /* Capable of multiple simulataneous operations */
    CSSM_BOOL SerialRequired;       /* Not capable of executing parallel operations */
    CSSM_CSP_TYPE CSPType;          /* Hardware/Software/Hybrid */
    CSSM_BOOL LoginRequired;        /* A login must be performed before use */
    uint32 SlotID;                  /* Slot reference associated with the token */
    char *SlotDescription;          /* Description of slot device */
    char *SlotVendor;               /* Manufacturer of the slot device */
    CSSM_BOOL SlotIsHardware;       /* The slot is hardware or virtual */
    CSSM_DATA ExclusiveCSPCertificate;
    char *Vendor;                   /* Manufacturer of the token */
    char *Description;              /* Description of the token */
    char *Label;                    /* Label associated with token */
    CSSM_DATA SerialNumber;         /* Serial number of device */
    CSSM_BOOL Removable;            /* Token can be removed from slot */
    CSSM_BOOL CapabilitiesInitialized;    /* Token available for use */
    uint32 NumberOfCapabilities;
    CSSM_CONTEXT_PTR Capabilities;
    CSSM_CSPPININFO PinInfo;        /* CSP Pin information */
    CSSM_CSPMEMINFO MemInfo;        /* CSP memory information */
    CSSM_CSPSESSIONINFO SessionInfo;    /* CSP multitasking information */
}CSSM_CSPINFO, *CSSM_CSPINFO_PTR;

#define CSSM_VALUE_NOT_AVAILABLE    (0xFFFFFFFF)

typedef struct cssm_tpinfo {
    uint32 VerMajor;
    uint32 VerMinor;
    CSSM_BOOL MultiTaskEnabled;
} CSSM_TPINFO, *CSSM_TPINFO_PTR;

typedef struct cssm_field {
    CSSM_OID FieldOid;
    CSSM_DATA FieldValue;
} CSSM_FIELD, *CSSM_FIELD_PTR;

typedef struct cssm_clinfo {
    CSSM_CERT_TYPE CertType;
    uint32 NumberOfFields;
    CSSM_OID_PTR CertTemplate;
    uint32 VerMajor;
    uint32 VerMinor;
    uint32 NumberOfTypes;
    CSSM_BOOL MultiTaskEnabled;
    CSSM_CERT_TYPE_PTR CertTranslationType;
} CSSM_CLINFO, *CSSM_CLINFO_PTR;

/* Static data associated with a data storage library add-in module */
typedef struct cssm_dlinfo{
    uint32 VerMajor;
    uint32 VerMinor;
    CSSM_DL_MODULE_TYPE DLModuleType;
    uint32 DeviceID; 
    CSSM_BOOL CapabilitiesInitialized;
    uint32 DeviceAccessFlags;
    CSSM_DATA ExclusiveDLMCertificate; 
    CSSM_BOOL LoginRequired;
    uint32 NumberOfRecordTypes;
    CSSM_DATA_RECORD_TYPE_PTR DataRecordTypes; 
    uint32 NumberOfAttributeUsageTypes;
    CSSM_DB_ATTRIBUTE_USAGE_PTR AttributeUsageTypes;
    uint32 NumberOfAttributeIdFormats;
    CSSM_ATTRIBUTE_ID_FORMAT_PTR AttributeIdFormats; 
    uint32 NumberOfRelOperatorTypes;
    CSSM_DB_OPERATOR_PTR RelOperatorTypes; 
    uint32 NumberOfConjOperatorTypes;
    CSSM_DB_CONJUNCTIVE_PTR ConjOperatorTypes;
    CSSM_DATA_PTR Reserved1;
}CSSM_DLINFO, *CSSM_DLINFO_PTR;


typedef CSSM_RETURN (*CSSM_NOTIFY_CALLBACK) (CSSM_CSP_HANDLE hCSP,
                                             uint32 Application,
                                             uint32 Reason,
                                             uint32 Param);

#define CSSM_NOTIFY_SURRENDER           0
#define CSSM_NOTIFY_COMPLETE            1
#define CSSM_NOTIFY_DEVICE_REMOVED      2
#define CSSM_NOTIFY_DEVICE_INSERTED     3

/* Structure for passing handle information to CSPs */
typedef struct cssm_handleinfo {
    uint32 SlotID;
    uint32 SessionFlags;
    CSSM_NOTIFY_CALLBACK Callback;
    uint32 ApplicationContext;
} CSSM_HANDLEINFO, *CSSM_HANDLEINFO_PTR;

#define CSSM_CSP_SESSION_EXCLUSIVE        0x0001 
#define CSSM_CSP_SESSION_READWRITE        0x0002
#define CSSM_CSP_SESSION_SERIAL           0x0004


#define CSSM_DL_STORE_ACCESS_SERIAL       CSSM_CSP_SESSION_SERIAL
#define CSSM_DL_STORE_ACCESS_EXCLUSIVE    CSSM_CSP_SESSION_EXCLUSIVE
#define CSSM_DL_STROE_ACCESS_READWRITE    CSSM_CSP_SESSION_READWRITE

/* structure for passing a memory function table to cssm */
typedef struct cssm_memory_funcs {
    void *(*malloc_func) (uint32 Size, void *AllocRef);
    void (*free_func) (void *MemPtr, void *AllocRef);
    void *(*realloc_func) (void *MemPtr, uint32 Size, void *AllocRef);
    void *(*calloc_func) (uint32 Num, uint32 Size, void *AllocRef);
    void *AllocRef;
} CSSM_MEMORY_FUNCS, *CSSM_MEMORY_FUNCS_PTR;

typedef CSSM_MEMORY_FUNCS CSSM_API_MEMORY_FUNCS;
typedef CSSM_API_MEMORY_FUNCS *CSSM_API_MEMORY_FUNCS_PTR;

/* structure for passing a memory function table to an add-in */
typedef struct cssm_spi_func_tbl {
    void *(*malloc_func) (CSSM_HANDLE AddInHandle, uint32 Size);
    void (*free_func) (CSSM_HANDLE AddInHandle, void *MemPtr);
    void  *(*realloc_func) (CSSM_HANDLE AddInHandle, void *MemPtr, uint32 Size);
    void  *(*calloc_func) (CSSM_HANDLE AddInHandle, uint32 Num, uint32 Size);
} CSSM_SPI_MEMORY_FUNCS, *CSSM_SPI_MEMORY_FUNCS_PTR;


/* Do not edit the following string. It should be expanded at check-in by version control. */
static const char sccs_id_INC_CSSMTYPE_H[] = { "@(#) $Archive: /cdsa/r1_2/inc/cssmtype.h $, $Revision: 6 $, $Modtime: 3/14/97 1:41p $" }; 

#endif /* _CSSMTYPE_H */
